#File type: <Function> set
#By Junxiang H., 2023/07/1
#wacmk.com/cn Tech. Supp.

#This script updates automaticly! Do not Modify!
import os
import numpy as np
try:
	from ShockFinder.Addon.Time import *
	import ShockFinder.Addon.Loader.FilesPreProcess as FilesPreProcess
	import ShockFinder.Addon.Loader.FileNamePreProcess as FileNamePreProcess
	from ShockFinder.Config import ShockFinderDir
	LoaderDir=os.path.join(ShockFinderDir,"Addon","Loader")
	AnalysisToolDir=os.path.join(ShockFinderDir,"Addon","AnalysisTool")
	PainterDir=os.path.join(ShockFinderDir,"Addon","Painter")
	IODir=os.path.join(ShockFinderDir,"Addon","IO")
	MultiprocessEngineDir=os.path.join(ShockFinderDir,"Addon","MultiprocessEngine")
	GUIDir=os.path.join(ShockFinderDir,"Addon","GUI")
except:
	from Addon.Time import *
	import Addon.Loader.FilesPreProcess as FilesPreProcess
	import Addon.Loader.FileNamePreProcess as FileNamePreProcess
	ShockFinderDir=""
	LoaderDir=os.path.join("Addon","Loader")
	AnalysisToolDir=os.path.join("Addon","AnalysisTool")
	PainterDir=os.path.join("Addon","Painter")
	IODir=os.path.join("Addon","IO")
	MultiprocessEngineDir=os.path.join("Addon","MultiprocessEngine")
	GUIDir=os.path.join("Addon","GUI")
LoaderPass=("__init__","Config","FileNamePreProcess","FilesPreProcess","LoaderModel")
AnalysisToolPass=("__init__","Config","AnalysisToolModel","Basic","TestData","Differential","Equilibrium","Mean","Radial","Harmonic","FastFourierTransform")
PainterPass=("__init__","Config")
IOPass=("__init__","Config","MySQL")
MultiprocessEnginePass=("__init__","Config")
GUIPass=("__init__","Config")

def get_header_end(modulename,main_fun_name="info"):
	Header="#File type: extension <Module> set\n#By Junxiang H., 2023/07/1\n#wacmk.com/cn Tech. Supp.\n\n#This script updates automaticly! Do not Modify!\n#Update time:"+now()+"\n\n"+modulename+"={}\n"
	End="if __name__==\"__main__\":\n\tprint(\"Testing Model:\",__file__)\n\tprint(\""+modulename+":\")\n\tfor i in "+modulename+".keys():\n\t\tprint(i,\":\","+modulename+"[i])\n\t\tprint(\"\\t\","+modulename+"[i]."+main_fun_name+")"
	return (Header,End)
def get_filesstr(modulename,moduledir,modulepass):
	FilesStr=""
	for i in tuple(set([FileNamePreProcess.FileNamePreProcess(i)[0] for i in FilesPreProcess.GetFiles(moduledir,"py")])-set(modulepass)):
		FilesStr+="try:\n\ttry:\n\t\timport ShockFinder.Addon."+modulename+"."+i+" as "+i+"\n\texcept Exception as err:\n\t\tprint(\""+i+"in ShockFinder dir importing error, reimporting in current dir\")\n\t\timport "+i+"\n\t"+modulename+"[\""+i+"\"]="+i+"\nexcept Exception as err:\n\tprint(\"Module: "+i+" import failure:\",err)\n\n" 
	return FilesStr
def Write(cdir,cname,header,body,end):
	filename=os.path.join(cdir,cname)
	file=open(filename,"w")
	file.writelines((header,body,end))
	file.close()

def Update(modulename,moduledir,modulepass,main_fun_name="info",cname="Config.py"):
	print("Update:",modulename,">>>",os.path.join(moduledir,cname))
	header,end=get_header_end(modulename,main_fun_name)
	Write(moduledir,cname,header,get_filesstr(modulename,moduledir,modulepass),end)
def Update_all():
	Update("Loader",LoaderDir,LoaderPass,"load")
	Update("AnalysisTool",AnalysisToolDir,AnalysisToolPass,"get")
	Update("Painter",PainterDir,PainterPass)
	Update("MultiprocessEngine",MultiprocessEngineDir,MultiprocessEnginePass,"build")
	Update("GUI",GUIDir,GUIPass,"show")
	Update("IO",IODir,IOPass)
if __name__=="__main__":
	Update_all()


#This is a model file for XUI page
#Junxiang H. 2023.07.09
import os
from XenonUI.XUIlib.imgtool import add_image
from XenonUI.XUIlib.page import *
logo=os.path.join("ShockFinder","Addon","GUI","XUIlib","image","Figure.png")

class page(page):
	img={
		"logo":logo
	}
	def page_menu1(self):	
		self.menu1=self.add_menu("menu1")
		self.add_item(self.menu1,self.ro_height)
		#...
	def page_menu2(self):	
		self.menu2=self.add_menu("menu2")
		self.add_item(self.menu1,self.ro_height)
		#...
	def initial(self):
		self.set_image(self.img["logo"])
		self.page_menu1()
		self.page_menu2()
	def show(self): #Default page
		self.tkobj.open_smb(None,self.fmbindex)
		self.tkobj.open_ro(None,self.fmbindex,self.menu1)


#File type: Basic Default Value and libraries, please modify with the situation
#By Junxiang H., 2023/07/1
#wacmk.com/cn Tech. Supp.

import os
ShockFinderVersion=7.2
ShockFinderDir=os.path.join(__file__.rsplit("ShockFinder",1)[0],"ShockFinder")
ShockFinderHeader=(
		'■■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■',
		'■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■■',
		'■■■■■■■■■□△■■■■■■■■■■■■■■■■■■■■■',
		'■■■■■■■■□■△■◁■■■■■■■ShockFinder v'+str(ShockFinderVersion)+'■■■■',
		'■■■■■■■□■■△■■■■■■■■■■■■■■■■■■■■■',
		'■■■■◁■□■■■■■■■■■■Junxiang H. &  C. B. Singh■■',
		'■■■■■■■□■■▽■■■■■■■■■■■■■■■■■■■■■',
		'■■■■■■■■□■▽■◁■■■■■wacmk.com/cn Tech. Supp.■■',
		'■■■■■■■■■□▽■■■■■■■■■■■■■■■■■■■■■',
		'■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■■',
		'■■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■'
	)
[print(i) for i in ShockFinderHeader]
print("\n")
#To Data Object
Default={
	"output":True,
	"logfile":None,
	"c":3.0e8,
	"mu":0.5,
	"k":1.38e-23,
	"mp":1.6726231e-27,
	"gamma":4/3
}
print("Loading general libraries...")
import ShockFinder.Addon.Time as Time
import ShockFinder.Addon.ConfigReader as ConfigReader
GeneralLib={
	"Time":Time,
	"ConfigReader":ConfigReader,
}

print("Loading multiprocessing engines library")
try:
	import ShockFinder.Addon.MultiprocessEngine.Config as MConfig
	MultiprocessEngine=MConfig.MultiprocessEngine
except Exception as err:
	print("Loading error:",err,"Please rebuild the config file")

print("Loading analysis tool libraries...")
import ShockFinder.Addon.AnalysisTool.Differential as Differential
import ShockFinder.Addon.AnalysisTool.Equilibrium as Equilibrium
import ShockFinder.Addon.AnalysisTool.Harmonic as Harmonic
import ShockFinder.Addon.AnalysisTool.Mean as Mean
import ShockFinder.Addon.AnalysisTool.Radial as Radial
import ShockFinder.Addon.AnalysisTool.TestData as TestData
import ShockFinder.Addon.AnalysisTool.FastFourierTransform as FastFourierTransform
AnalysisLib={
	"Differential":Differential,
	"Gradient":Differential.gradient,
	"Divergence":Differential.divergence,
	"Harmonic":Harmonic.get,
	"Harmonic_src":Harmonic,
	"Equilibrium":Equilibrium,
	"Mean":Mean.get,
	"Mean_src":Mean,
	"Radial":Radial.get,
	"Radial_src":Radial,
	"TestData":TestData,
	"FFT":FastFourierTransform.get
}

print("Loading analysis tools...")
try:
	import ShockFinder.Addon.AnalysisTool.Config as AConfig
	AnalysisTool=AConfig.AnalysisTool
except Exception as err:
	print("Loading error:",err,"Please rebuild the config file")

print("Loading file loader libraries...")
import ShockFinder.Addon.Loader.FilesPreProcess as FPP
import ShockFinder.Addon.Loader.FileNamePreProcess as FNPP
LoaderLib={
	"FPP":FPP,
	"FNPP":FNPP
}

print("Loading file loaders...")
try:
	import ShockFinder.Addon.Loader.Config as LConfig
	Loader=LConfig.Loader
except Exception as err:
	print("Loading error:",err,"Please rebuild the config file")

print("Loading figure painters...")
try:
	import ShockFinder.Addon.Painter.Config as PConfig
	Painter=PConfig.Painter
except Exception as err:
	print("Loading error:",err,"Please rebuild the config file")

print("Loading IO engines...")
try:
	import ShockFinder.Addon.IO.Config as IConfig
	IO=IConfig.IO
except Exception as err:
	print("Loading error:",err,"Please rebuild the config file")

print("Loading GUI...")
try:
	import ShockFinder.Addon.GUI.Config as GConfig
	GUI=GConfig.GUI
except Exception as err:
	print("Loading error:",err,"Please rebuild the config file")

try:
	Config={
		"GeneralLib":GeneralLib,
		"MultiprocessEngine":MultiprocessEngine,
		"AnalysisLib":AnalysisLib,
		"AnalysisTool":AnalysisTool,
		"LoaderLib":LoaderLib,
		"Loader":Loader,
		"Painter":Painter,
		"IO":IO,
		"GUI":GUI
	}
except Exception as err:
	print("Config setup error:",err,"Please rebuild the config file")


#File type: <Function> set
#By Junxiang H., 2023/07/2
#wacmk.com/cn Tech. Supp.

import datetime,time
def now():
	return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

def get_time():
	return time.time()

def sleep(sec):
	time.sleep(sec)


#This is a model file for the Loader Addon
#It will be created when creating a new Loader
#Note!!!!: places which marke "<>" have to be updated, and delete "<>".

#File type: <Function> return a new <Object: Data>
#By Junxiang H., 2023/07/01
#wacmk.com/cn Tech. Supp.

#if you would like to import some packages,
#during the data loading.
#Please put that packages into this folder and using:

try:
	from ShockFinder.Addon.AnalysisTool.Basic import *
	#if AvgTh_CAL is True
	#import ShockFinder.Addon.AnalysisTool.Mean as Mean
	#import ShockFinder.Addon.AnalysisTool.<packages name> as <packages name>
except:
	from Basic import *
	#import Mean #debug
	#import <packages name>

need=[]
#args will be inserted into Data Object
#vargs will not be inserted into Data Object
import numpy as np
def get(Dataobj,args={},vargs={}):
	Dataobj.quantities.update(args)
	for i in need:
		if i not in Dataobj.quantities.keys() and i not in vargs.keys():
			print("Warning: args:",i,"is needed without definding")
			return Dataobj
	quantities={
		#operation with dict args
		#...
		<quantity name>:... #update here
	}
	Dataobj.quantities.update(quantities)
	return Dataobj
def result(quantity_name=None,anafname=None):
	return () #this function will return result types shown in GUI
#if AvgTh mode is needed, please set AvgTh_cal=True
#The below code can be ignored, if set to False
AvgTh_cal=False
def get_AvgTh(Dataobj,args={},vargs={"Mean_axis":(1,)}):
	try:
		if AvgTh_cal:
			import copy
			newneed=copy.copy(need)
			if "Mean_axis" not in newneed:
				newneed.append("Mean_axis")
			Dataobj.quantities.update(args)
			for i in newneed:
				if i not in Dataobj.quantities.keys() and i not in vargs.keys():
					print("Warning: args:",i,"is needed without definding")
					return Dataobj
			meanstr=""
			try:
				meanaxis=vargs["Mean_axis"]
			except:
				meanaxis=Dataobj.quantities["Mean_axis"]
			for i in meanaxis:
				meanstr+=str(i)+"@"+str((round(vargs["Mean_axis"+str(i)][0],2),round(vargs["Mean_axis"+str(i)][1],2)))+"_" if "Mean_axis"+str(i) in vargs.keys() else meanstr+=str(i)+"@"+str((round(Dataobj.quantities["Mean_axis"+str(i)][0],2),round(Dataobj.quantities["Mean_axis"+str(i)][1],2)))+"_" if "Mean_axis"+str(i) in Dataobj.quantities.keys() else ""
			quantities={"AvgTh_"+meanstr+"<quantity name>":...} #update here
			Dataobj.quantities.update(quantities)
		else:
			print("Warning: AvgTh mode is not opened: <quantity name>") #update here
	except:
		print("Warning: AvgTh mode is not definded:", __file__) #update here
	return Dataobj

if __name__=="__main__":
	print("Testing Model:",__file__)
	from TestData import TestData
	TestData=get(TestData)
	print("Testing Result:", TestData.quantities[<quantity name>]) #update here!
	if AvgTh_cal:
		TestData=get_AvgTh(TestData)
		print("Testing Result:", TestData.quantities["AvgTh_<quantity name>"]) #update here!


#Example.py
import os
try:
	from ShockFinder.Addon.GUI.XUIlib.XUIConfig import Pages as Pages #those folders/packages path modified by yourself
except:
	pass

from XenonUI import XUI
from XenonUI.XUIlib.SystemInfo import SystemInfo
try:
	from ShockFinder.Config import ShockFinderDir
	config_file=os.path.join(ShockFinderDir,"Addon","GUI","XUIlib","XUIConfig.py")
except:
	config_file=os.path.join("XUIlib","XUIConfig.py")
pages_import="ShockFinder.Addon.GUI.XUIlib.pages" #aa.bb.cc.pages


def CollectPages():
	pages_dir=os.path.join(*pages_import.split("."))
	import datetime
	pages_except=("__init__","page_model")
	def FileNamePreProcess(filename):
		filedir,filename=os.path.split(filename)
		fileindex,filetype=os.path.splitext(filename)
		return (fileindex.split(".")[-1],filedir,filetype.split(".")[1])
	def GetFiles(filedir=os.getcwd(),filetype=""):
		result=()
		for file in os.listdir(filedir):
			if file.endswith(filetype):
				result+=(os.path.join(filedir,file),)
		return result
	strc="#File type: extension <class page> set\n#By Junxiang H., 2023/07/9\n#wacmk.com/cn Tech. Supp.\n\n#This script updates automaticly! Do not Modify!\n#Update time:"+datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")+"\n\nPages={}\n"
	try:
		vaild_pages=tuple(set([FileNamePreProcess(i)[0] for i in GetFiles(pages_dir,"py")])-set(pages_except))
	except:
		vaild_pages=tuple(set([FileNamePreProcess(i)[0] for i in GetFiles(os.path.join("XUIlib","pages"),"py")])-set(pages_except))
	for i in vaild_pages:
		strc+="import "+pages_import+"."+i+" as "+i+"\n"+"Pages[\""+i+"\"]="+i+"\n"
	if "Help" not in vaild_pages:
		strc+="import XenonUI.XUIlib.pages.Help as Help\nPages[\"Help\"]=Help\n"
	if "Exit" not in vaild_pages:
		strc+="import XenonUI.XUIlib.pages.Exit as Exit\nPages[\"Exit\"]=Exit\n"
	file=open(config_file,"w")
	file.writelines(strc)
	file.close()
def show(**args):
	class XUIOBJ:
		def __init__(self,**args):
			try:
				from ShockFinder.Addon.GUI.XUIlib.image.config import SF7img_encode
				SF7img_encode()
			except:
				pass
			try:
				from ShockFinder.Addon.GUI.XUIlib.XUIConfig import Pages as Pages #those folders/packages path modified by yourself
			except Exception as err:
				CollectPages()
				print("XUI error:",err)
				print("XUI reconfigurates completed, please rerun ShockFinder again")
				exit(0)
			args.update({"XUIOBJ":self})
			self.guiobj=XUI()
			self.guiobj.build(**args)
			#systeminfo
			self.systeminfo=SystemInfo(self.guiobj)
			self.systeminfo.lunch(1)
			self.Pages={}
			for i in reversed(Pages.keys()):
				if i not in ("Exit","Help"):
					self.Pages[i]=Pages[i].page(self.guiobj,**args)
			if "Help" in Pages.keys():
				self.Pages["Help"]=Pages["Help"].page(self.guiobj,**args)
			if "Exit" in Pages.keys():
				self.Pages["Exit"]=Pages["Exit"].page(self.guiobj,**args)
			self.iofun=self.guiobj.io_recv
	return XUIOBJ(**args)

if __name__=="__main__":
	show=os.path.exists(config_file)
	CollectPages()
	if show:
		xui=build() #you can set some arguments in here as you need
		xui.guiobj.show()
	else:
		print("XUI configurates completed, please rerun script again")


#File type: extension <Module> set
#By Junxiang H., 2023/07/1
#wacmk.com/cn Tech. Supp.

#This script updates automaticly! Do not Modify!
#Update time:2023-10-24 06:22:13

Painter={}
try:
	try:
		import ShockFinder.Addon.Painter.Surface as Surface
	except Exception as err:
		print("Surfacein ShockFinder dir importing error, reimporting in current dir")
		import Surface
	Painter["Surface"]=Surface
except Exception as err:
	print("Module: Surface import failure:",err)

try:
	try:
		import ShockFinder.Addon.Painter.P2D as P2D
	except Exception as err:
		print("P2Din ShockFinder dir importing error, reimporting in current dir")
		import P2D
	Painter["P2D"]=P2D
except Exception as err:
	print("Module: P2D import failure:",err)

try:
	try:
		import ShockFinder.Addon.Painter.Basic as Basic
	except Exception as err:
		print("Basicin ShockFinder dir importing error, reimporting in current dir")
		import Basic
	Painter["Basic"]=Basic
except Exception as err:
	print("Module: Basic import failure:",err)

try:
	try:
		import ShockFinder.Addon.Painter.Line as Line
	except Exception as err:
		print("Linein ShockFinder dir importing error, reimporting in current dir")
		import Line
	Painter["Line"]=Line
except Exception as err:
	print("Module: Line import failure:",err)

try:
	try:
		import ShockFinder.Addon.Painter.Scatter as Scatter
	except Exception as err:
		print("Scatterin ShockFinder dir importing error, reimporting in current dir")
		import Scatter
	Painter["Scatter"]=Scatter
except Exception as err:
	print("Module: Scatter import failure:",err)

try:
	try:
		import ShockFinder.Addon.Painter.P3D as P3D
	except Exception as err:
		print("P3Din ShockFinder dir importing error, reimporting in current dir")
		import P3D
	Painter["P3D"]=P3D
except Exception as err:
	print("Module: P3D import failure:",err)

if __name__=="__main__":
	print("Testing Model:",__file__)
	print("Painter:")
	for i in Painter.keys():
		print(i,":",Painter[i])
		print("\t",Painter[i].info)


import os
try:
	from ShockFinder.Config import ShockFinderDir
	SFXUIlib=os.path.join(ShockFinderDir,"Addon","GUI","XUIlib")
	SFXUIImagelib=os.path.join(SFXUIlib,"image")
	Image_SF7=os.path.join(SFXUIImagelib,"Main_logo.png")
except:
	SFXUIImagelib=""
	Image_SF7="Main_logo.png"

def SF7img_decode():
	try:
		os.rename(Image_SF7,os.path.join(SFXUIImagelib,"Main_logo.py"))
	except:
		pass

def SF7img_encode():
	try:
		os.rename(os.path.join(SFXUIImagelib,"Main_logo.py"),Image_SF7)
	except:
		pass

if __name__=="__main__":
	SF7img_decode()


import h5py
import os
import numpy as np

class ShockFinderFiguresHDF5:
    def __init__(self,output=print):
        self.filename = None
        self.output=output
    def set_file(self, filename):
        self.filename = filename
        if not self.is_valid_hdf5():
            self.create_new_hdf5()

    def is_valid_hdf5(self,filename=None):
        if filename==None:
            filename=self.filename
        if filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return False

        try:
            with h5py.File(filename, 'r') as f:
                if "HDF5LABEL" in f.attrs and f.attrs["HDF5LABEL"] == "ShockFinderSavedFile":
                    return True
        except:
            pass
        return False

    def create_new_hdf5(self):
        with h5py.File(self.filename, 'w') as f:
            f.attrs["HDF5LABEL"] = "ShockFinderSavedFile"

            # Create primary groups
            f.create_group("Data")
            f.create_group("Units")
            configurations = f.create_group("Configurations")
            
            # Create subgroups for Data
            for dimension in ["0D", "1D", "2D", "3D"]:
                f.create_group(f"Data/{dimension}")

            # Create subgroups for Configurations
            config_2d = configurations.create_group("2D")
            config_3d = configurations.create_group("3D")
            
            # Second-level subgroups for Configurations/2d
            for subgroup in ["TimeSequency", "FFT", "Lines", "Surfaces"]:
                config_2d.create_group(subgroup)

            # Second-level subgroups for Configurations/3d
            for subgroup in ["TimeSequency", "Lines", "Surfaces", "Scatter"]:
                config_3d.create_group(subgroup)


    def read_units(self, group_name=None):
        if self.filename==None:
            self.output("Error: no vaild hdf5 file selected")
            return {}
        data = {}
        with h5py.File(self.filename, 'r') as f:
            units_group = f["Units"]
            
            def read_subgroup(subgroup):
                subgroup_data = {}
                for key in subgroup.keys():
                    value = subgroup[key][()]
                    if isinstance(value, bytes):
                        value = value.decode()
                    subgroup_data[key] = value
                return subgroup_data
            
            if group_name:
                if group_name in units_group:
                    return read_subgroup(units_group[group_name])  # Return a one-level dictionary
                self.output(f"Unit '{group_name}' does not exist.")
                return {}

            return tuple(units_group.keys()) #only return keys
            for subgroup_name in units_group.keys():
                data[subgroup_name] = read_subgroup(units_group[subgroup_name])
                    
        return data

    def write_units(self, data_dict):
        if self.filename==None:
            self.output("Error: no vaild hdf5 file selected")
            return None
        with h5py.File(self.filename, 'a') as f:
            units_group = f.require_group("Units")
            
            for subgroup_name, subgroup_data in data_dict.items():
                subgroup = units_group.require_group(subgroup_name)
                for key, value in subgroup_data.items():
                    if key in subgroup:
                        del subgroup[key]
                    subgroup.create_dataset(key, data=value)

    def del_units(self, group_name):
        if self.filename==None:
            self.output("Error: no vaild hdf5 file selected")
            return None
        with h5py.File(self.filename, 'a') as f:  # Open the file in append mode
            units_group = f["Units"]
            if group_name in units_group:  # Check if the subgroup exists
                del units_group[group_name]  # Delete the subgroup
            else:
                self.output(f"Unit '{group_name}' does not exist.")

    def read_config(self, secondary, tertiary, subgroup_name=None):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return {}

        with h5py.File(self.filename, 'r') as f:
            group_path = f"Configurations/{secondary}/{tertiary}"

            # Helper function to read a subgroup's datasets and return them as a dictionary
            def read_subgroup(subgroup):
                data = {}
                for key in subgroup.keys():
                    value = subgroup[key][()]
                    if isinstance(value, bytes):
                        value = value.decode()
                    data[key] = value
                return data

            # If subgroup_name is specified, return its datasets as a one-level dictionary
            if subgroup_name:
                if subgroup_name in f[group_path]:
                    return read_subgroup(f[f"{group_path}/{subgroup_name}"])
                else:
                    self.output(f"Configuration '{subgroup_name}' does not exist.")
                    return {}  # Return an empty dictionary if the subgroup doesn't exist

            return tuple(f[group_path].keys()) #only return keys
            # If subgroup_name is not specified, return all data as a two-level dictionary
            data = {}
            for sub_name in f[group_path].keys():
                data[sub_name] = read_subgroup(f[f"{group_path}/{sub_name}"])

            return data


    def write_config(self, secondary, tertiary, data_dict):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return 

        with h5py.File(self.filename, 'a') as f:
            group_path = f"Configurations/{secondary}/{tertiary}"
            target_group = f.require_group(group_path)

            # Iterate over subgroups in the two-level dictionary
            for subgroup_name, subgroup_data in data_dict.items():
                subgroup = target_group.require_group(subgroup_name)
                
                # Write datasets for each subgroup
                for key, value in subgroup_data.items():
                    if key in subgroup:
                        del subgroup[key]
                    subgroup.create_dataset(key, data=value)

    def del_config(self, secondary, tertiary, subgroup_name_to_delete=None):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return

        with h5py.File(self.filename, 'a') as f:
            group_path = f"Configurations/{secondary}/{tertiary}"
            
            if subgroup_name_to_delete:
                if subgroup_name_to_delete in f[group_path]:
                    del f[f"{group_path}/{subgroup_name_to_delete}"]
                else:
                    self.output(f"Configuration '{subgroup_name_to_delete}' does not exist.")
                    return {}  # Return an empty dictionary if the subgroup doesn't exist
            else:
                for subgroup in list(f[group_path].keys()):  
                    del f[f"{group_path}/{subgroup}"]

    def read_data(self, dimension, subgroup=None):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return {}

        with h5py.File(self.filename, 'r') as f:
            if f"Data/{dimension}" not in f:
                return
            target_group = f[f"Data/{dimension}"]
            
            # If subgroup is specified
            if subgroup:
                if subgroup in target_group:
                    subgroup_data = target_group[subgroup]
                    data = {}
                    for key in subgroup_data.keys():
                        value = subgroup_data[key][()]
                        if isinstance(value, bytes):
                            value = value.decode()
                        data[key] = value
                    return data
                self.output(f"Data '{subgroup}' does not exist.")
                return {}  # Return an empty dictionary if the subgroup doesn't exist

            # If subgroup is not specified, return all data as a two-level dictionary
            return tuple(target_group.keys()) #only return keys
            data = {}
            for sub_name in target_group.keys():
                subgroup_data = target_group[sub_name]
                dataset_dict = {}
                for key in subgroup_data.keys():
                    value = subgroup_data[key][()]
                    if isinstance(value, bytes):
                        value = value.decode()
                    dataset_dict[key] = value
                data[sub_name] = dataset_dict

            return data

    def write_data(self, dimension, data_dict):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return

        with h5py.File(self.filename, 'a') as f:
            for subgroup, datasets in data_dict.items():
                group_path = f"Data/{dimension}/{subgroup}"
                subgroup = f.require_group(group_path)
                
                for key, value in datasets.items():
                    if key in subgroup:
                        del subgroup[key]
                    subgroup.create_dataset(key, data=value)


    def read_data(self, dimension, subgroup=None):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return {}

        with h5py.File(self.filename, 'r') as f:
            data = {}

            if subgroup:  # If a specific subgroup is given
                if subgroup not in f[f"Data/{dimension}"]:
                    self.output(f"Data '{subgroup}' does not exist under 'Data/{dimension}'.")
                    return {}
                target_group = f[f"Data/{dimension}/{subgroup}"]
                for key in sorted(target_group.keys(), key=int):  # Sort keys as integers
                    value = target_group[key][()]
                    if isinstance(value, bytes):
                        value = value.decode()
                    data[key] = value
                return data
            return tuple(f[f"Data/{dimension}"].keys())
            main_group = f[f"Data/{dimension}"]
            for subgroup_name in sorted(main_group.keys(), key=int):  # Sort keys as integers
                target_group = main_group[subgroup_name]
                subgroup_data = {}
                for key in sorted(target_group.keys(), key=int):  # Sort keys as integers
                    value = target_group[key][()]
                    if isinstance(value, bytes):
                        value = value.decode()
                    subgroup_data[key] = value
                data[subgroup_name] = subgroup_data

            return data


    def del_data(self, dimension, dataset_name_to_delete=None):
        if self.filename is None:
            self.output("Error: no vaild hdf5 file selected")
            return

        with h5py.File(self.filename, 'a') as f:
            group_path = f"Data/{dimension}"

            if dataset_name_to_delete:
                if dataset_name_to_delete in f[group_path]:
                    del f[f"{group_path}/{dataset_name_to_delete}"]
                else:
                    self.output(f"Data '{dataset_name_to_delete}' does not exist under '{group_path}'.")
            else:
                for dataset in list(f[group_path].keys()):
                    del f[f"{group_path}/{dataset}"]


#File type: Fixed <Class>
#By Junxiang H., 2023/07/03
#wacmk.com/cn Tech. Supp.

#Fixed files do not modify
import ShockFinder.Data as Data
import ShockFinder.Addon.Time as Time
def get_par(args,name,default=None):
	try:
		return args[name]
	except:
		return default
def DBC_Findindex(databases,tind):
	for i in databases:
		try:
			return (i,i.tindex.index(tind))
		except:
			continue
	return None
def DataBaseConnect(*databases):
	print(*(["Connecting databases:\n"]+["\t"+str(i+1)+":\t"+str(databases[i])+"\n" for i in range(len(databases))]))
	newdb=DataBase(databases[0].infomation,databases[0].printfun)
	for i in range(max([max(j.tindex) for j in databases])+1):
		result=DBC_Findindex(databases,i)
		if result!=None:
			db,index=result
			newdb.data.append(db.data[index])
			newdb.tindex.append(db.tindex[index])
		else:
			newdb.data.append(None)
			newdb.tindex.append(-1)
	return newdb
class DataBase:
	data=[]
	infomation={}
	tindex=[]
	anaf=[]
	vargs=[]
	resultnames=[]
	loader=None
	filesname=None
	def __init__(self,infomation,printfun=print):
		self.infomation=infomation
		self.printfun=printfun
	def load_data(self,loader,**filesname):
		#for filename in filesname:
		#	self.data.append(Data(*loader.load(filename)))
		#	self.data[-1].update(self.infomation)
		self.loader=loader
		self.filesname={int(i):filesname[i] for i in filesname.keys()}
		self.tindex=list(filesname.keys())
	def update_infomation(self,**infomation):
		self.infomation.update(infomation)
		[i.update(self.infomation) for i in self.data]
	def input_default(self):
		self.data[-1].update(self.infomation)
	def drop_buffer(self,*index):
		for ind in range(len(self.data)) if len(index)==0 else index:
			self.data[ind]=None 
	def update(self,*index,output=True,**quantities):
		if len(index)==0:
			index=range(len(self.data))
		if output:
			self.printfun("Updating parameters:",quantities)
		for ind in index:
			try:
				self.data[ind].update(quantities)
			except:
				self.data.append(None)
				if output:
					print("Warning: index",ind,"exceed, creating...")
				if len(self.data)==ind+1:
					self.data[ind]=Data.Data({},quantities)
					self.tindex.append(ind)
	def update_with_index(self,**quantities): #2d quantities
		for key in quantities.keys():
			self.update(int(key),output=False,**quantities[key])
	def analysis_data(self,analyze_fun,*index,**vargs):
		if len(index)==0:
			index=range(len(self.data))
		info=get_par(vargs,"info")
		qtname=get_par(vargs,"quantity_name")
		resultname=get_par(vargs,"result")
		for ind in index:
			kwargs={
				"Dataobj":self.data[ind]
			}
			if "quantity_name" in analyze_fun.__code__.co_varnames:
				kwargs["quantity_name"]=qtname
			if "vargs" in analyze_fun.__code__.co_varnames:
				kwargs["vargs"]=vargs
			self.printfun("Analyzing \"",info,"\" with \"",resultname,"\" in index:",self.tindex[ind],"...")
			self.data[ind]=analyze_fun(**kwargs)
	def save(self,*index,**vargs):
		IOfun=get_par(vargs,"IOfun")
		newrsn=[]
		for i in self.resultnames:
			if type(i) not in (list,tuple):
				newrsn.append(i)
			else:
				for j in i:
					newrsn.append(j)
		if IOfun!=None:
			filename=get_par(vargs,"filename",str(int(Time.get_time())))
			if len(index)==0:
				index=range(len(self.data))
			for ind in index:
				self.printfun("Saving results @ index:",self.tindex[ind],"...")
				pp=list(self.data[ind].grid.keys())+newrsn+["geometry"]
				IOfun(filename,self.data[ind],self.tindex[ind],*pp)
				DropBuffer=get_par(vargs,"DropBuffer",True)
				if DropBuffer:
					self.drop_buffer(ind)
	def save_analyze_pars(self,anaf,**vargs):
		self.anaf.append(anaf)
		self.vargs.append(vargs)
		self.resultnames.append(get_par(vargs,"result"))
	def analyze(self):
		if self.loader==None:#test mode
			for i in range(len(self.data)):
				for j in range(len(self.anaf)):
					self.analysis_data(self.anaf[j],i,**self.vargs[j])
				self.save(i,**self.vargs[j])
		else: #file mode
			for i in self.filesname.keys():
				self.printfun("Loading file \"",self.filesname[i],"\"...")
				self.data.append(self.loader(self.filesname[i]))
				self.input_default()
				for j in range(len(self.anaf)):
					self.analysis_data(self.anaf[j],len(self.data)-1,**self.vargs[j]) 
				self.save(len(self.data)-1,**self.vargs[j])
	def check_quantities(self,quantities=[],*index):
		if len(index)==0:
			index=range(len(self.data))
		if type(quantities) not in (list,tuple):
			quantities=[quantities]
		for quantity in quantities:
			self.printfun("Checking quantity: \"",quantity,"\"...")
			for ind in index:
				try:
					self.data[ind].quantities[quantity]
				except:
					try:
						self.printfun("Error: Quantity: \"",quantity,"\" not exists...",color="red")
					except:
						self.printfun("Error: Quantity: \"",quantity,"\" not exists...")
					return False
		return True
	def get_dimension(self,quantity_name="rho",index=-1):
		return self.data[index].quantities[quantity_name].ndim
	def get_geometry(self,index=-1):
		return self.data[index].quantities["geometry"]
	def build_test_data(self,testdatafun,num):
		self.data=testdatafun(num)
		self.update_infomation()


#File type: algroithm <Function> set
#By Junxiang H., 2023/07/01
#wacmk.com/cn Tech. Supp.

try:
	from ShockFinder.Addon.AnalysisTool.Basic import *
except:
	from Basic import *
import numpy as np
def GetMeanAxis(meanaxis=(1,2)):
	return np.flip(np.sort(meanaxis))

def GetMeanRegion(Dataobj,meanaxis,vargs={}): #True Value
	meanaxisregion=[]
	for i in meanaxis:
		try:
			meanaxisregion.append(vargs["Mean_axis"+str(i)])
		except:
			try:
				meanaxisregion.append(Dataobj.quantities["Mean_axis"+str(i)])
			except:
				meanaxisregion.append([Dataobj.grid["x"+str(i+1)][0],Dataobj.grid["x"+str(i+1)][-1]])
	return meanaxisregion

def GetMeanRegionIndex(Dataobj,meanaxis,meanaxisregion):
	index=[]
	for i in range(len(meanaxis)):
		index.append([np.where(Dataobj.grid["x"+str(meanaxis[i]+1)]>=meanaxisregion[i][0])[0][0],np.where(Dataobj.grid["x"+str(meanaxis[i]+1)]<=meanaxisregion[i][1])[0][-1]+1])
	return index

def GetMeanRadialIndex(Dataobj,meanaxis,radial):
	index=[]
	for i in range(len(meanaxis)):
		if radial[i]==None:
			ind=len(Dataobj.grid["x"+str(meanaxis[i]+1)])/2
			if int(ind)==ind:
				index.append([int(ind)-1,int(ind)+1])
			else:
				index.append([int(ind)+1,int(ind)+2])
		elif radial[i]>=min(Dataobj.grid["x"+str(meanaxis[i]+1)]) and radial[i]<=max(Dataobj.grid["x"+str(meanaxis[i]+1)]):
			if radial[i] in Dataobj.grid["x"+str(meanaxis[i]+1)]:
				ind=np.where(Dataobj.grid["x"+str(meanaxis[i]+1)]==radial[i])[0][0]
				index.append([ind,ind+1])
			else:
				index.append([np.where(np.where(Dataobj.grid["x"+str(meanaxis[i]+1)]<=radial[i])[0][-1],Dataobj.grid["x"+str(meanaxis[i]+1)]>=radial[i])[0][0]+1])
		else:
			index.append([-1,-1])
	return index

def GetMean(quantity,meanaxis,meanindex):
	new_quantity=quantity
	try: #3d
		for i in range(len(meanaxis)):
			if meanaxis[i]==0:
				new_quantity=new_quantity[meanindex[i][0]:meanindex[i][1],:,:]
			elif meanaxis[i]==1:
				new_quantity=new_quantity[:,meanindex[i][0]:meanindex[i][1],:]
			elif meanaxis[i]==2:
				new_quantity=new_quantity[:,:,meanindex[i][0]:meanindex[i][1]]
	except:#2d
		for i in range(len(meanaxis)):
			if meanaxis[i]==0:
				new_quantity=new_quantity[meanindex[i][0]:meanindex[i][1],:]
			elif meanaxis[i]==1:
				new_quantity=new_quantity[:,meanindex[i][0]:meanindex[i][1]]
	for i in meanaxis:
		new_quantity=np.mean(new_quantity,i)
	return new_quantity

def Mean(Dataobj,quantity_name,vargs={"Mean_axis":(1,)}):
	meanaxis=get_par(Dataobj,vargs,"Mean_axis",Dataobj.quantities[quantity_name].ndim-1)
	if type(meanaxis) not in (list,tuple,np.ndarray):
		meanaxis=(meanaxis,)
	meanaxis=GetMeanAxis(meanaxis)
	meanregion=GetMeanRegion(Dataobj,meanaxis,vargs)
	meanindex=GetMeanRegionIndex(Dataobj,meanaxis,meanregion)
	return GetMean(Dataobj.quantities[quantity_name],meanaxis,meanindex)

def get(Dataobj,quantity_name,args={},vargs={"Mean_axis":(1,)}):
	if type(quantity_name) in (np.ndarray,list,tuple):
		for i in quantity_name:
			Dataobj=get(Dataobj,i,args,vargs)
		return Dataobj
	need=[quantity_name]
	Dataobj.quantities.update(args)
	for i in need:
		if i not in Dataobj.quantities.keys() and i not in vargs.keys():
			print("Warning: args:",i,"is needed without definding")
			return Dataobj
	meanstr=""
	meanaxis=get_par(Dataobj,vargs,"Mean_axis",Dataobj.quantities[quantity_name].ndim-1)
	if type(meanaxis) not in (list,tuple,np.ndarray):
		meanaxis=(meanaxis,)
	for i in meanaxis:
		if "Mean_axis"+str(i) in vargs.keys():
			meanstr+=str(i)+"@"+str((round(vargs["Mean_axis"+str(i)][0],2),round(vargs["Mean_axis"+str(i)][1],2)))+"_"
		elif "Mean_axis"+str(i) in Dataobj.quantities.keys():
			meanstr+=str(i)+"@"+str((round(Dataobj.quantities["Mean_axis"+str(i)][0],2),round(Dataobj.quantities["Mean_axis"+str(i)][1],2)))+"_"
	quantities={
		"Mean_"+meanstr+quantity_name:Mean(Dataobj,quantity_name,vargs)
	}
	Dataobj.quantities.update(quantities)
	return Dataobj

def result(quantity_name=None,anafname=None):
	return ["Mean_"+i for i in quantity_name.split(",")]

if __name__=="__main__":
	print("Testing Model:",__file__)
	from TestData import TestData
	TestData=get(TestData,"rho")
	print("Testing Result:", TestData.quantities["Mean_rho"]) #update 



#File type: extension <Module> set
#By Junxiang H., 2023/07/1
#wacmk.com/cn Tech. Supp.

#This script updates automaticly! Do not Modify!
#Update time:2023-10-24 06:22:13

MultiprocessEngine={}
try:
	try:
		import ShockFinder.Addon.MultiprocessEngine.XME as XME
	except Exception as err:
		print("XMEin ShockFinder dir importing error, reimporting in current dir")
		import XME
	MultiprocessEngine["XME"]=XME
except Exception as err:
	print("Module: XME import failure:",err)

if __name__=="__main__":
	print("Testing Model:",__file__)
	print("MultiprocessEngine:")
	for i in MultiprocessEngine.keys():
		print(i,":",MultiprocessEngine[i])
		print("\t",MultiprocessEngine[i].build)


#File type: <Function> return a new <Object: Data>
#By Junxiang H., 2023/07/01
#wacmk.com/cn Tech. Supp.

#if you would like to import some packages,
#during the data loading.
#Please put that packages into this folder and using:

try:
	import ShockFinder.Addon.AnalysisTool.Mean as Mean
	import ShockFinder.Addon.AnalysisTool.Differential as Differential
	from ShockFinder.Addon.AnalysisTool.Basic import *
except:
	import Mean #debug
	import Differential
	from Basic import *
import numpy as np
need=["rho","geometry","Temperature","c","mp","gamma"]
def get(Dataobj,args={},vargs={}):
	Dataobj.quantities.update(args)
	for i in need:
		if i not in Dataobj.quantities.keys() and i not in vargs.keys():
			print("Warning: args:",i,"is needed without definding")
			return Dataobj
	quantities={"Bremsstrahlung":Dataobj.quantities["rho"]**2*Dataobj.quantities["Temperature"]**0.5*2*np.pi*1.4e-27/Dataobj.quantities["mp"]**2}
	Dataobj.quantities.update(quantities)
	if Dataobj.quantities["geometry"] in ("SPHERICAL","POLAR"):
		if Dataobj.quantities["geometry"]=="SPHERICAL":
			intfun=Differential.integrate_sph_body
		else:
			intfun=Differential.integrate_pol_body
	else:
		intfun=Differential.integrate_body
	INTRMAX=None
	if "INTRMAX" in args.keys():
		INTRMAX=args["INTRMAX"].split("$")[1]
	elif "INTRMAX" in vargs.keys():
		INTRMAX=vargs["INTRMAX"].split("$")[1]
	if INTRMAX==None:
		INTRMAX=Dataobj.grid["x1"]
		nc=Dataobj.quantities["Bremsstrahlung"]
	else:
		try:
			try:
				Index=Mean.GetMeanRegionIndex(Dataobj,(0,),((0,Dataobj.quantities[INTRMAX]),))[0][1]
			except:
				Index=Mean.GetMeanRegionIndex(Dataobj,(0,),((0,Dataobj.grid[INTRMAX]),))[0][1]
			nc=Dataobj.quantities["Bremsstrahlung"][:Index]
			INTRMAX=Dataobj.grid["x1"][:Index]
		except:
			print("Warning:",INTRMAX,"decode error")
			INTRMAX=Dataobj.grid["x1"]
			nc=Dataobj.quantities["Bremsstrahlung"]
	if Dataobj.quantities["Bremsstrahlung"].ndim==3:#3d
		quantities={"Total_Bremsstrahlung":intfun(nc,INTRMAX,Dataobj.grid["x2"],Dataobj.grid["x3"])}
	elif Dataobj.quantities["Bremsstrahlung"].ndim==2:#2d
		quantities={"Total_Bremsstrahlung":intfun(nc,INTRMAX,Dataobj.grid["x2"])}
	elif Dataobj.quantities["Bremsstrahlung"].ndim==1:#1d
		quantities={"Total_Bremsstrahlung":intfun(nc,INTRMAX)}
	Dataobj.quantities.update(quantities)
	return Dataobj
def result(quantity_name=None,anafname=None):
	return ("Bremsstrahlung","Total_Bremsstrahlung")


if __name__=="__main__":
	print("Testing Model:",__file__)
	from TestData import TestData
	import Temperature
	TestData=Temperature.get(TestData)
	TestData=get(TestData)
	print("Testing Result:", TestData.quantities["Bremsstrahlung"])
	print(Differential.integrate_sph_body(TestData.quantities["Bremsstrahlung"],TestData.grid["x1"],TestData.grid["x2"]))


# coding=utf-8
#File type: <Function> set
#By Junxiang H., 2023/07/5
#wacmk.com/cn Tech. Supp.
try:
	import ShockFinder.Addon.Time as Time
except:
	import datetime
	class Time:
		def now():
			return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
import numpy as np,pandas as pd, pymysql
Filetype="sql"
def get_save_sql(data,index,tablename,*quantities_name):
	dt={}
	sql="insert into `"+tablename+"` (`index`) values ("+str(index)+");\nupdate `"+tablename+"` set "
	for i in quantities_name:
		if i in data.quantities.keys():
			sql+="`"+i+"`=\""+Formatdata(data.quantities[i])+"\","
	sql=sql[:-1]+" where `index`="+str(index)+";\n"
	return sql
def PreSave(filename,*quantities_name,**databaseinfo):
	filename=filename.rsplit(".",1)[0]
	sql="#■■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■\n#■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■■\n#■■■■■■■■■□△■■■■■■■■■■■■■■■■■■■■■\n#■■■■■■■■□■△■◁■■■■■■■ShockFinder v7.0■■■■\n#■■■■■■■□■■△■■■■■■■■■■■■■■■■■■■■■\n#■■■■◁■□■■■■■■■■■■Junxiang H. &  C. B. Singh■■\n#■■■■■■■□■■▽■■■■■■■■■■■■■■■■■■■■■\n#■■■■■■■■□■▽■◁■■■■■wacmk.com/cn Tech. Supp.■■\n#■■■■■■■■■□▽■■■■■■■■■■■■■■■■■■■■■\n#■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■■\n#■■■■■■■■■■■□■■■■■■■■■■■■■■■■■■■■\n\n#Project Name: "+filename+"\n#Build at: "+Time.now()+"\n"
	sql+="insert into `mapping_database`(`name`) values (\""+filename+"\");\nupdate `mapping_database` set `parameters`=\""
	for i in databaseinfo.keys():
		sql+=i+"="+str(databaseinfo[i]).replace("[","").replace("]","").replace("(","").replace(")","").replace("\\","\\\\")+";"
	sql=sql[:-1]+"\" where `name`=\""+filename+"\"\n"
	sql+="create table if not exists `"+filename+"`(`index` int NOT NULL, PRIMARY KEY (`index`));\n"
	for i in quantities_name:
		sql+="alter table `"+filename+"` add column `"+i+"` longtext NULL;\n"
	file=open(filename,"w")
	file.writelines(sql)
	file.close()
	return sql #debug
def Save(filename,data,index,*quantities_name):
	filename=filename.rsplit(".",1)[0]
	sql=get_save_sql(data,index,filename,*quantities_name)
	file=open(filename,"a")
	file.writelines(sql)
	file.close()
def Load(filename,mysqlobj):
	pass
def Formatdata(dt): #max 2d
	if dt.ndim==1:#grid or 1d data:
		strc=str(tuple(dt.tolist())).replace(" ","").replace("\"","").replace("\'","")
	elif dt.ndim==2:#2d data:
		strc=str(tuple([tuple(i.tolist()) for i in dt])).replace(" ","").replace("\"","").replace("\'","")
	elif dt.ndim==3:#3d data:
		strc=str(tuple([tuple([tuple(j.tolist()) for j in i]) for i in dt ])).replace(" ","").replace("\"","").replace("\'","")
	return strc
def Decodedata(strc):
	dt=str_to_array(strc)
	if dt.ndim==1:#1d
		dt=np.array([float(i) for i in dt])
	elif dt.ndim==2:
		dt=np.array([np.array([float(j) for j in i]) for i in dt])
	else:
		dt=np.array([np.array([np.array([float(k) for k in j]) for j in i]) for i in dt])
	return dt
def char_num(strc,char):
	num=0
	for i in strc:
		if i==char:
			num+=1
	return num
def str_to_array(strc):
	if type(strc) is not str:
		return strc
	strc=strc.replace(" ","")
	if strc=="" or not char_num(strc,"]")==char_num(strc,"[") or not char_num(strc,")")==char_num(strc,"(") or not (strc[0]=="[" and strc[-1]=="]" or strc[0]=="(" and strc[-1]==")"):
		return strc
	pt=0
	ptb=[]
	pte=[]
	ptstr=""
	anb=""
	ane=""
	tmpresult=[]
	result=[]
	tmp=strc.split(",")
	for i in tmp:
		if pt<=1 and (i[0]=="[" and i[-1]=="]" and char_num(i,"]")==char_num(i,"[") or i[0]=="(" and i[-1]==")" and char_num(i,"(")==char_num(i,")")):
			sttp=str_to_array(i[1:-1])
			if len(tmp)==1:
				anb=i[0]
				ane=i[-1]
				if sttp!="":
					result.append(sttp)
			else:
				if i[0]=="[":
					if sttp!="":
						result.append([str_to_array(i[1:-1])])
					else:
						result.append([])
				else:
					if sttp!="":
						result.append((str_to_array(i[1:-1]),))
					else:
						result.append(())
		else:
			for j in i:
				if j=="[":
					ptb.append(j)
					if anb=="":
						anb=j
						ane="]"
					pte.append("]")
					pt+=1
					if pt>1:
						ptstr+=j
				elif j=="(":
					ptb.append(j)
					if anb=="":
						anb=j
						ane=")"
					pte.append(")")
					pt+=1
					if pt>1:
						ptstr+=j
				elif j==pte[pt-1]:
					del pte[pt-1]
					del ptb[pt-1]
					pt-=1
					if pt>0:
						ptstr+=j
				else:
					ptstr+=j
			if pt>1:
				ptstr+=","
			elif pt<=1 and ptstr!="":
				result.append(str_to_array(ptstr))
				ptstr=""
	if anb=="(" and ane==")":
		return np.array(result)
	return result
class MySQL:
	host="127.0.0.1"
	user="pluto_data_stream_1"
	password="xx1234"
	port=3306
	database="data_stream_1"
	charset="utf8"
	conn=0
	cursor=0
	is_connect=False
	def connect(self):
		if not self.is_connect:
			try:
				self.conn=pymysql.connect(host=self.host,user=self.user,passwd=self.password,port=self.port,db=self.database,charset=self.charset)
				self.cursor=self.conn.cursor()
				self.build_mapping_tables()
				self.is_connect=True
				return True
			except Exception as err:
				print(err)
				self.is_connect=False
				return False
		return True
	def reconnect(self):
		self.close()
		return self.connect()
	def close(self):
		if self.is_connect:
			self.conn.close()
			self.is_connect=False
	def execute(self,sql):
		try:
			result=self.cursor.execute(sql.replace("\\","\\\\"))
			if int(result)>0:
				return True
			return False
		except Exception as err:
			print(err)
			return err
	def get_result(self,sql):
		status=self.execute(sql)
		if str(status)=="True":
			return self.cursor.fetchall()
		elif str(status)=="False":
			return None
		return None
	def build_mapping_tables(self):
		sql="create table if not exists `mapping_database`(  `name` varchar(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL, `parameters` longtext NULL, `time` datetime NULL DEFAULT CURRENT_TIMESTAMP, PRIMARY KEY (`name`) USING BTREE) engine=innodb CHARACTER SET = utf8 COLLATE = utf8_general_ci ROW_FORMAT = Dynamic;\n"
		sql+="create table if not exists `mapping_figure`(  `name` varchar(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL, `type` varchar(255) CHARACTER SET utf8 COLLATE utf8_general_ci NULL, `time` datetime NULL DEFAULT CURRENT_TIMESTAMP, PRIMARY KEY (`name`) USING BTREE) engine=innodb CHARACTER SET = utf8 COLLATE = utf8_general_ci ROW_FORMAT = Dynamic;\n"
		if exce:
			self.execute(sql)
		return sql
	def savefigure(self,fig,figinfo):
		#use pd
		pass
	def load(self,tablename):
		pass

if __name__=="__main__":
	'''
	import time
	dt=np.random.random((512,280))
	print(PreSave("testtb","test1","test2","test3","test4"))
	t1=time.time()
	dst=Formatdata(dt)
	t2=time.time()
	print(dt,"\nFormattime=",t2-t1)
	t1=time.time()
	dt=Decodedata(dst)
	t2=time.time()
	print(dt,"\nDecodeTime=",t2-t1)
	'''
	pass


#File type: <Function> return a new <Object: Data>
#By Junxiang H., 2023/07/02
#wacmk.com/cn Tech. Supp.

#if you would like to import some packages,
#during the data loading.
#Please put that packages into this folder and using:

try:
	from ShockFinder.Addon.AnalysisTool.Basic import *
	import ShockFinder.Addon.AnalysisTool.Mean as Mean
	#import ShockFinder.Addon.AnalysisTool.<packages name>
except:
	from Basic import *
	import Mean #debug
	#import <packages name>
import copy
import numpy as np
need=[]
#args will be inserted into Data Object
#vargs will not be inserted into Data Object
#ShockDeduct, Shock_x1, Shock_x2, Shock_x3
def get(Dataobj,quantity_name,args={},vargs={"ShockDeduct":0.1}):
	if type(quantity_name) in (np.ndarray,list,tuple):
		for i in quantity_name:
			Dataobj=get(Dataobj,i,args,vargs)
		return Dataobj
	Dataobj.quantities.update(args)
	newneed=copy.deepcopy(need)
	newneed+=["Divergence_"+quantity_name]
	for i in newneed:
		if i not in Dataobj.quantities.keys() and i not in vargs.keys():
			print("Warning: args:",i,"is needed without definding")
			return Dataobj
	quantities={}
	div=copy.deepcopy(Dataobj.quantities["Divergence_"+quantity_name])
	shockx1=get_par(Dataobj,vargs,"Shock_bou_x1")
	if shockx1==None:
		shockx1=[Dataobj.grid["x1"][0],Dataobj.grid["x1"][-1]]
	shockx1ind=Mean.GetMeanRegionIndex(Dataobj,(0,),(shockx1,))[0]
	quantities["Shock_x1"]=Dataobj.grid["x1"][shockx1ind[0]:shockx1ind[1]]
	if Dataobj.quantities[quantity_name].ndim>=2:
		shockx2=get_par(Dataobj,vargs,"Shock_bou_x2")
		if shockx2==None:
			shockx2=[Dataobj.grid["x2"][0],Dataobj.grid["x2"][-1]]
		shockx2ind=Mean.GetMeanRegionIndex(Dataobj,(1,),(shockx2,))[0]
		quantities["Shock_x2"]=Dataobj.grid["x2"][shockx2ind[0]:shockx2ind[1]]
		if Dataobj.quantities[quantity_name].ndim==3:
			shockx3=get_par(Dataobj,vargs,"Shock_bou_x3")
			if shockx3==None:
				shockx3=[Dataobj.grid["x3"][0],Dataobj.grid["x3"][-1]]
			shockx3ind=Mean.GetMeanRegionIndex(Dataobj,(2,),(shockx3,))[0]
			quantities["Shock_x3"]=Dataobj.grid["x3"][shockx3ind[0]:shockx3ind[1]]
			div=div[shockx1ind[0]:shockx1ind[1],shockx2ind[0]:shockx2ind[1],shockx3ind[0]:shockx3ind[1]]
		else:
			div=div[shockx1ind[0]:shockx1ind[1],shockx2ind[0]:shockx2ind[1]]
	else:
		div=div[shockx1ind[0]:shockx1ind[1]]
	sdu=get_par(Dataobj,vargs,"ShockDeduct",0.1)
	sign=get_par(Dataobj,vargs,"Shockway","Positive")
	if sign=="Positive":
		div-=np.max(div)*sdu
		div[np.where(div<0)]=0
	else:
		div-=np.min(div)*sdu
		div[np.where(div>0)]=0
		div=-div
	quantities["Shock_"+quantity_name]=div
	if quantities["Shock_"+quantity_name].ndim==1:#1d
		quantities["Shock_Outermost_x1_"+quantity_name]=-1 if len(np.where(quantities["Shock_"+quantity_name]!=0)[0])==0 else Dataobj.grid["x1"][np.where(quantities["Shock_"+quantity_name]!=0)[0][-1]]
	elif quantities["Shock_"+quantity_name].ndim==2:#2d
		quantities["Shock_Outermost_x1_"+quantity_name]=np.array([-1 if len(np.where(i!=0)[0])==0 else Dataobj.grid["x1"][np.where(i!=0)[0][-1]] for i in quantities["Shock_"+quantity_name].T]) #default
		quantities["Shock_Outermost_x2_"+quantity_name]=np.array([-1 if len(np.where(i!=0)[0])==0 else Dataobj.grid["x2"][np.where(i!=0)[0][-1]] for i in quantities["Shock_"+quantity_name]])
	elif quantities["Shock_"+quantity_name].ndim==3:
		quantities["Shock_Outermost_x1_"+quantity_name]=np.array([np.array([-1 if len(np.where(j!=0)[0])==0 else Dataobj.grid["x1"][np.where(j!=0)[0][-1]] for j in i.T]) for i in quantities["Shock_"+quantity_name].T])
		quantities["Shock_Outermost_x2_"+quantity_name]=np.array([np.array([-1 if len(np.where(j!=0)[0])==0 else Dataobj.grid["x2"][np.where(j!=0)[0][-1]] for j in i.T]) for i in quantities["Shock_"+quantity_name]])
		quantities["Shock_Outermost_x3_"+quantity_name]=np.array([np.array([-1 if len(np.where(j!=0)[0])==0 else Dataobj.grid["x3"][np.where(j!=0)[0][-1]] for j in i]) for i in quantities["Shock_"+quantity_name]])

	Dataobj.quantities.update(quantities)
	return Dataobj
def result(quantity_name=None,anafname=None):
	return ["Shock_"+i for i in quantity_name.split(",")]+["Shock_Outermost_x1_"+i for i in quantity_name.split(",")]
#if AvgTh mode is needed, please set AvgTh_cal=True
#The below code can be ignored, if set to False
AvgTh_cal=False
def get_AvgTh(Dataobj,args={},vargs={"Mean_axis":(1,)}):
	try:
		if AvgTh_cal:
			newneed=copy.deepcopy(need)
			if "Mean_axis" not in newneed:
				newneed.append("Mean_axis")
			Dataobj.quantities.update(args)
			for i in newneed:
				if i not in Dataobj.quantities.keys() and i not in vargs.keys():
					print("Warning: args:",i,"is needed without definding")
					return Dataobj
			meanstr=""
			try:
				meanaxis=vargs["Mean_axis"]
			except:
				meanaxis=Dataobj.quantities["Mean_axis"]
			for i in meanaxis:
				if "Mean_axis"+str(i) in vargs.keys():
					meanstr+=str(i)+"@"+str((round(vargs["Mean_axis"+str(i)][0],2),round(vargs["Mean_axis"+str(i)][1],2)))+"_"
				elif "Mean_axis"+str(i) in Dataobj.quantities.keys():
					meanstr+=str(i)+"@"+str((round(Dataobj.quantities["Mean_axis"+str(i)][0],2),round(Dataobj.quantities["Mean_axis"+str(i)][1],2)))+"_"
			quantities={"AvgTh_"+meanstr+"<quantity name>":...} #update here
			Dataobj.quantities.update(quantities)
		else:
			print("Warning: AvgTh mode is not opened: <quantity name>") #update here
	except:
		print("Warning: AvgTh mode is not definded:", __file__) #update here
	return Dataobj

if __name__=="__main__":
	print("Testing Model:",__file__)
	from TestData import TestData
	import Differential,math
	TestData=Differential.divergence(TestData,"rho")
	TestData=get(TestData,"rho",vargs={"ShockDeduct":0.1,"Shock_bou_x1":(2,9),"Shock_bou_x2":(0.1*np.pi,0.9*np.pi)})
	print("Testing Result:", TestData.quantities["Shock_rho"]) #update here!
	from matplotlib import pyplot as plt
	plt.figure("imshow",facecolor="lightgray")
	plt.imshow(TestData.quantities["Shock_rho"],cmap="RdBu",extent=[TestData.quantities["Shock_x1"][0],TestData.quantities["Shock_x1"][-1],TestData.quantities["Shock_x2"][0],TestData.quantities["Shock_x2"][-1]])
	plt.colorbar()
	plt.xticks(TestData.quantities["Shock_x1"])
	plt.yticks(TestData.quantities["Shock_x2"])
	plt.show()
	if AvgTh_cal:
		TestData=get_AvgTh(TestData)
		print("Testing Result:", TestData.quantities["AvgTh_<quantity name>"]) #update here!


import setuptools
 
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
 
setuptools.setup(
    name="ShockFinder",
    version="7.2.11",
    author="Junxiang H. & C. B. Singh",
    author_email="huangjunxiang@mail.ynu.edu.cn",
    description="ShockFinder is an interactive scientic simulation data analysis software based on python and supports multi-process, multi-mode, I/O access and drawing.",
    long_description=long_description, 
    long_description_content_type="text/markdown",
    url="https://github.com/wacmkxiaoyi/Shockfinder", 
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',  
)


#File type: <Function> return a new <Object: Data>
#By Junxiang H., 2023/07/01
#wacmk.com/cn Tech. Supp.

#Spherical Only!!!!
#Support 2D (xoz plane only), 3D

#if you would like to import some packages,
#during the data loading.
#Please put that packages into this folder and using:
try:
	from ShockFinder.Addon.AnalysisTool.Basic import *
except:
	from Basic import *
from scipy.special import lpmv
import numpy as np
import math
need=[]
#args will be inserted into Data Object
#vargs will not be inserted into Data Object
def cal_true_angle_th(angle,angle_begin,angle_end):
	return (angle-angle_begin)/(angle_end-angle_begin)*math.pi
def cal_true_angle_phi(angle,angle_begin,angle_end):
	return 2*cal_true_angle_th(angle,angle_begin,angle_end)
def get(Dataobj,quantity_name,args={},vargs={"Harmonic_Max_l":3,"Harmonic_Max_m":3}):
	if type(quantity_name) in (np.ndarray,list,tuple):
		for i in quantity_name:
			Dataobj=get(Dataobj,i,args,vargs)
		return Dataobj
	Dataobj.quantities.update(args)
	for i in need+[quantity_name]:
		if i not in Dataobj.quantities.keys() and i not in vargs.keys():
			print("Warning: args:",i,"is needed without definding")
			return Dataobj
	hm_lmax=get_par(Dataobj,vargs,"Harmonic_Max_l",3)
	hm_mmax=get_par(Dataobj,vargs,"Harmonic_Max_m",3)
	if Dataobj.quantities[quantity_name].ndim>=2:
		angle_begin_th=get_par(Dataobj,vargs,"Harmonic_Th_beg")
		angle_begin_th=angle_begin_th if angle_begin_th!=None else Dataobj.grid["x2"][0]
		angle_end_th=get_par(Dataobj,vargs,"Harmonic_Th_end")
		angle_end_th=angle_end_th if angle_end_th!=None else Dataobj.grid["x2"][-1]
	else:
		print("Warning: Harmonic analysis only supports 2D (xoz plane) or 3D")
		return Dataobj
	if Dataobj.quantities[quantity_name].ndim==3:
		angle_begin_phi=get_par(Dataobj,vargs,"Harmonic_Phi_beg")
		angle_begin_phi=angle_begin_phi if angle_begin_phi!=None else Dataobj.grid["x3"][0]
		angle_end_phi=get_par(Dataobj,vargs,"Harmonic_Phi_end")
		angle_end_phi=angle_end_phi if angle_end_phi!=None else Dataobj.grid["x3"][-1]
	quantities={}
	trueangle=cal_true_angle_th(Dataobj.grid["x2"],angle_begin_th,angle_end_th)
	trueangle=trueangle.reshape((1,len(trueangle)))
	if Dataobj.quantities[quantity_name].ndim==3:
		trueangle=trueangle.reshape((1,len(trueangle),1))
		trueanglephi=cal_true_angle_phi(Dataobj.grid["x3"],angle_begin_phi,angle_end_phi)
		trueanglephi=trueanglephi.reshape((1,1,len(trueanglephi)))
	for l in range(hm_lmax+1):
		if Dataobj.quantities[quantity_name].ndim==2: #2d
			quantities["Harmonic_l@"+str(l)+"_"+quantity_name]=(l+1)*Dataobj.quantities[quantity_name]*lpmv(0,l,np.cos(trueangle))
		elif Dataobj.quantities[quantity_name].ndim==3: #3d
			for m in range(max(-hm_mmax,-l),min(hm_mmax+1,l+1)):
				ct=(2*l+1)*math.factorial(l-abs(m))/(4*math.pi*math.factorial(l+abs(m)))
				quantities["Harmonic_l@"+str(l)+"_m@"+str(m)+"_"+quantity_name]=Dataobj.quantities[quantity_name]*math.e**(m*trueanglephi*1j)*ct*lpmv(m,l,np.cos(trueangle))
	Dataobj.quantities.update(quantities)
	return Dataobj
def result(quantity_name=None,anafname=None):
	return ["Harmonic_l@?_"+i for i in quantity_name.split(",")]+["Harmonic_l@?_m@?_"+i for i in quantity_name.split(",")]


if __name__=="__main__":
	print("Testing Model:",__file__)
	from TestData import TestData
	TestData=get(TestData,"rho")
	print("Testing Result:", TestData.quantities["Harmonic_l@3_rho"])


#This is a model file for the Loader Addon
#It will be created when creating a new Loader
#Note!!!!: places which marke "<>" have to be updated, and delete "<>".

#File type: <Function> return a new <Object: Data>
#By Junxiang H., 2023/07/01
#wacmk.com/cn Tech. Supp.

#if you would like to import some packages,
#during the data loading.
#Please put that packages into this folder and using:

try:
	#if AvgTh_CAL is True
	import ShockFinder.Addon.AnalysisTool.Mean as Mean
	from ShockFinder.Addon.AnalysisTool.Basic import *
except:
	import Mean #debug
	from Basic import *
	
import numpy as np
need=[]
#args will be inserted into Data Object
#vargs will not be inserted into Data Object
def get(Dataobj,quantity_name,args={},vargs={"Radial_axis":(1,),"Radial_value":(None,)}):
	if type(quantity_name) in (np.ndarray,list,tuple):
		for i in quantity_name:
			Dataobj=get(Dataobj,i,args,vargs)
		return Dataobj
	need=[quantity_name]
	Dataobj.quantities.update(args)
	for i in need:
		if i not in Dataobj.quantities.keys() and i not in vargs.keys():
			print("Warning: args:",i,"is needed without definding")
			return Dataobj
	radialaxis=get_par(Dataobj,vargs,"Radial_axis",Dataobj.quantities[quantity_name].ndim-1)
	if type(radialaxis) not in (list,tuple,np.ndarray):
		radialaxis=(radialaxis,)
	radialvalue=get_par(Dataobj,vargs,"Radial_value")
	if type(radialvalue) not in (list,tuple,np.ndarray):
		radialvalue=(radialvalue,)
	radialaxis=Mean.GetMeanAxis(radialaxis)
	radialindex=Mean.GetMeanRadialIndex(Dataobj,radialaxis,radialvalue)

	radial_str=""
	for i in range(len(radialaxis)):
		if radialvalue[i]!=None:
			radial_str+=str(radialaxis[i])+"@"+str(round(radialvalue[i],2))+"_"
	quantities={
		"Radial_"+radial_str+quantity_name:Mean.GetMean(Dataobj.quantities[quantity_name],radialaxis,radialindex)
	}
	Dataobj.quantities.update(quantities)
	return Dataobj
def result(quantity_name=None,anafname=None):
	return ["Radial_"+i for i in quantity_name.split(",")]

def get_boundary(Dataobj,quantity_name):
	if Dataobj.quantities[quantity_name].ndim==3:#3d
		x1=(Dataobj.quantities[quantity_name][:1,:,:],Dataobj.quantities[quantity_name][-1:,:,:])
		x2=(Dataobj.quantities[quantity_name][:,:1,:],Dataobj.quantities[quantity_name][:,-1:,:])
		x3=(Dataobj.quantities[quantity_name][:,:,:1],Dataobj.quantities[quantity_name][:,:,-1:])
		return (x1,x2,x3)
	elif Dataobj.quantities[quantity_name].ndim==2: #2d
		x1=(Dataobj.quantities[quantity_name][:1,:],Dataobj.quantities[quantity_name][-1:,:])
		x2=(Dataobj.quantities[quantity_name][:,:1],Dataobj.quantities[quantity_name][:,-1:])
		return (x1,x2)
	else:#1d
		x1=(Dataobj.quantities[quantity_name][0],Dataobj.quantities[quantity_name][-1])
		return (x1,)

if __name__=="__main__":
	print("Testing Model:",__file__)
	from TestData import TestData
	TestData=get(TestData,"rho")
	print("Testing Result:", TestData.quantities["Radial_rho"]) #update 


#File type: <Function> return dict
#By Junxiang H., 2023/07/02
#wacmk.com/cn Tech. Supp.

#if you would like to import some packages,
#during the data loading.
#Please put that packages into this folder and using:


try:
	from ShockFinder.Addon.AnalysisTool.Basic import *
	import ShockFinder.Addon.AnalysisTool.Differential as Differential
	#if AvgTh_CAL is True
	#import ShockFinder.Addon.AnalysisTool.Mean
	#import ShockFinder.Addon.AnalysisTool.<packages name>
except:
	from Basic import *
	import Differential
	#import Mean #debug
	#import <packages name>
import numpy as np
from scipy.optimize import fsolve
import math
#rs gravitational type: rs=1 2GM/c**2, rs=2 GM/c**2
#args will be inserted into Data Object
#vargs will not be inserted into Data Object
c=3e8
mu=0.5
k=1.38e-23
mp=1.6726231e-27
G=6.67259e-11
Msun=1.989e30
et=6.7e-29

def Equilibrium(rb,lam,mach,gamma=4/3,rs=1,eng=0,theta_inj=0,steps=10000):
	def cal_vR(sintheta):
		return ((2*mach**2*(gamma-1)/(mach**2*(gamma-1)*sintheta**2+2))*(eng+rs/(2*(rb-rs))-lam**2*math.cos(theta_inj)**2/2/rb**2/sintheta**2))**0.5
	def cal_theta(i):
		sintheta=i[0]
		return rb**2*(1-sintheta**2)-2*rb*sintheta*(rb*sintheta-rs)**2/rs**2*cal_vR(sintheta)**2/mach**2*gamma
	quantities={}
	quantities["Accretion_x2_beg"]=math.asin(fsolve(cal_theta,[1])[0])
	quantities["Accretion_x2_end"]=math.pi-quantities["Accretion_x2_beg"]
	psc=np.array([cal_vR(math.sin(i)) for i in np.arange(quantities["Accretion_x2_beg"],quantities["Accretion_x2_end"],(quantities["Accretion_x2_end"]-quantities["Accretion_x2_beg"])/steps)])
	quantities["Accretion_vR"]=np.array([psc for i in range(steps)])
	quantities["Accretion_InjFlux"]=Differential.integrate_sph_sur(quantities["Accretion_vR"],np.arange(0,rb,rb/steps),th=np.arange(quantities["Accretion_x2_beg"],quantities["Accretion_x2_end"],(quantities["Accretion_x2_end"]-quantities["Accretion_x2_beg"])/steps),rr=("max",),surface=("r",))[0]
	quantities["Accretion_Min_Tem"]=cal_vR(math.sin(math.pi/2))**2*c**2*mu*mp/mach**2/gamma/k
	quantities["rs"]=rs
	return quantities

def SphericalGrid(rbeg,rend,tbeg,tend,rgnum):
	def cal_detr(i):
		detr=i[0]
		return [
		detr/rbeg-2*((rend/(rbeg+detr))**(1/(rgnum-1))-1)/((rend/(rbeg+detr))**(1/(rgnum-1))+1)
		]
	def cal_ptr(detr,detphi):
		return abs(math.log(rend/(rbeg+detr),10)-rgnum*math.log((2+detphi)/(2-detphi),10))
	detr=fsolve(cal_detr,[0.02])[0]
	detphi=detr/rbeg
	div=(tend-tbeg)/detphi
	result=0
	if div!=int(div):
		div_a=int(div)
		detphi_a=(tend-tbeg)/div_a
		detr_a=rbeg*detphi_a
		div_b=int(div)+1
		detphi_b=(tend-tbeg)/div_b
		detr_b=rbeg*detphi_b
		if cal_ptr(detr_a,detphi_a)>cal_ptr(detr_b,detphi_b):
			result=(detr_b,div_b)
		else:
			result=(detr_a,div_a)
	else:
		result=(detr,div)
	quantities={
		"r_beg":rbeg,
		"r1":result[0],
		"r_end":rend,
		"r_grid_num":rgnum,
		"th_beg":tbeg,
		"th_end":tend,
		"th_grid_num":result[1],
		"grid_error":cal_ptr(result[0],(tend-tbeg)/result[1])
	}
	return quantities

def Unit(mass,injflux,injrho,hequantities,basic_m=Msun,basic_r=2*G/c**2,basic_v=c):
	quantities={}
	quantities["unit_r"]=mass*basic_r*basic_m/hequantities["rs"]
	quantities["unit_t"]=quantities["unit_r"]/basic_v
	quantities["unit_rho"]=injflux/injrho/hequantities["Accretion_InjFlux"]/quantities["unit_r"]**2/basic_v
	quantities["unit_rho"]*=0.001 #to g/cm**3
	quantities["unit_r"]*=100 #to cm
	return quantities

def EdditionMassFlux_to_MassFlux(massflux,rb,unit_r): #to g/s
	return massflux*rb*unit_r*mp*c/et

def YearMassFlux_to_MassFlux(massflux,basic_m=Msun):
	return massflux*Msun/86400/365

if __name__=="__main__":
	print("Testing Model:",__file__)
	#from TestData import TestData
	#TestData=get(TestData)
	#print("Testing Result:", TestData.quantities[<quantity name>]) #update here!
	eq=Equilibrium(200,1.75,5)
	print(eq)
	print(SphericalGrid(2,200,0.1*math.pi,0.9*math.pi,512))
	print(Unit(10,YearMassFlux_to_MassFlux(1e-11),1,eq))


#This is a model file for XUI page
#Junxiang H. 2023.07.09
import os,copy
from XenonUI.XUIlib.imgtool import add_image
from XenonUI.XUIlib.page import *
import ShockFinder.Addon.Time as Time
from tkinter import *
from tkinter import ttk,filedialog
from multiprocessing import cpu_count
class page(page):
	img={
		"logo":Image_A
	}
	pars={}
	parmax=50
	def get(self):
		pass
	def GS_MPE(self):
		MPE=self.add_menu("Multi-process",submenu=1)
		self.pars["MultiprocessEngine"]={}
		#Multiprocessing
		self.add_row(MPE) #skip row (empty row)
		self.add_title(MPE,"Multi-process")
		Label(self.add_row(MPE,bx=150),text="="*500).place(x=0,y=0,anchor="nw")  #begin
		box=self.add_row(MPE,bx=150)
		Label(box,text="Engine",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		egs=list(self.pageargs["Infobj"].Config["MultiprocessEngine"].keys())+[None]
		MPE_ENG=ttk.Combobox(box,width=18,height=len(egs) if len(egs)<=10 else 10,values=egs)
		if self.pageargs["Infobj"].Default_MPE in self.pageargs["Infobj"].Config["MultiprocessEngine"]:
			MPE_ENG.set(self.pageargs["Infobj"].Default_MPE)
		else:
			MPE_ENG.set(str(egs[0]))
		MPE_ENG.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_mpeeng(event):
			self.pars["MultiprocessEngine"]["Engine"]=MPE_ENG.get()
			self.tkobj.io_recv("Updated MultiprocessEngine-Engine to",self.pars["MultiprocessEngine"]["Engine"])
		self.pars["MultiprocessEngine"]["Engine"]=MPE_ENG.get()
		button_mpeeng=Button(box,text="Update",width=5)
		button_mpeeng.bind("<ButtonRelease>",fun_mpeeng)
		button_mpeeng.pack(side="left")
		Label(self.add_row(MPE,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(MPE,bx=150)
		Label(box,text="Cores Num",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		MPE_PNUM=Entry(box,width=20)
		MPE_PNUM.insert(0,cpu_count() if cpu_count()<=4 and cpu_count()>1 else 4 if cpu_count()>4 else 1)
		MPE_PNUM.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_MPE_PNUM(event):
			self.pars["MultiprocessEngine"]["pnum"]=int(MPE_PNUM.get())
			self.tkobj.io_recv("Updated MultiprocessEngine-pnum to",self.pars["MultiprocessEngine"]["pnum"])
		self.pars["MultiprocessEngine"]["pnum"]=int(MPE_PNUM.get())
		button_MPE_PNUM=Button(box,text="Update",width=5)
		button_MPE_PNUM.bind("<ButtonRelease>",fun_MPE_PNUM)
		button_MPE_PNUM.pack(side="left")
		Label(self.add_row(MPE,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(MPE,bx=150)
		Label(box,text="Use log",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		do_with_log=ttk.Combobox(box,width=18,height=2,values=[True,False])
		do_with_log.set(str(True))
		do_with_log.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_button_do_with_log(event):
			self.pars["MultiprocessEngine"]["do_with_log"]=True if do_with_log.get()=="True" else False
			self.tkobj.io_recv("Updated MultiprocessEngine-do_with_log to",self.pars["MultiprocessEngine"]["do_with_log"])
		self.pars["MultiprocessEngine"]["do_with_log"]=True if do_with_log.get()=="True" else False
		button_do_with_log=Button(box,text="Update",width=5)
		button_do_with_log.bind("<ButtonRelease>",fun_button_do_with_log)
		button_do_with_log.pack(side="left")
		Label(self.add_row(MPE,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(MPE,bx=150)
		Label(box,text="Log file",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		logfile=Entry(box,width=15)
		logfile.insert(0,str(None))
		logfile.pack(side="left")
		def fun_logfile(event):
			folder_path=filedialog.asksaveasfilename()
			if folder_path!="":
				logfile.delete(0,"end")
				logfile.insert(0,folder_path)
		button=Button(box,text="Select",width=5)
		button.bind("<ButtonRelease>",fun_logfile)
		button.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_logfile(event):
			if logfile.get()!="":
				self.pars["MultiprocessEngine"]["logfile"]=None if logfile.get()=="None" else logfile.get()
				self.tkobj.io_recv("Updated MultiprocessEngine-logfile to",self.pars["MultiprocessEngine"]["logfile"])
		self.pars["MultiprocessEngine"]["logfile"]=None if logfile.get()=="None" else logfile.get()
		button_logfile=Button(box,text="Update",width=5)
		button_logfile.bind("<ButtonRelease>",fun_logfile)
		button_logfile.pack(side="left")
		Label(self.add_row(MPE,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(MPE,bx=150)
		Label(box,text="Show MPE info",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		show_version_info=ttk.Combobox(box,width=18,height=2,values=[True,False])
		show_version_info.set(str(True))
		show_version_info.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_show_version_info(event): 
			self.pars["MultiprocessEngine"]["show_version_info"]=True if show_version_info.get()=="True" else False
			self.tkobj.io_recv("Updated MultiprocessEngine-show_version_info to",self.pars["MultiprocessEngine"]["show_version_info"])
		button_show_version_info=Button(box,text="Update",width=5)
		button_show_version_info.bind("<ButtonRelease>",fun_show_version_info)
		button_show_version_info.pack(side="left")
		self.pars["MultiprocessEngine"]["show_version_info"]=True if show_version_info.get()=="True" else False
		Label(self.add_row(MPE,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(MPE,bx=150)
		Label(box,text="Show log in screen",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		print_in_screen=ttk.Combobox(box,width=18,height=2,values=[True,False])
		print_in_screen.set(str(True))
		print_in_screen.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_sprint_in_screen(event): 
			self.pars["MultiprocessEngine"]["print_in_screen"]=True if print_in_screen.get()=="True" else False
			self.tkobj.io_recv("Updated MultiprocessEngine-print_in_screen to",self.pars["MultiprocessEngine"]["print_in_screen"])
		self.pars["MultiprocessEngine"]["print_in_screen"]=True if print_in_screen.get()=="True" else False
		button_print_in_screen=Button(box,text="Update",width=5)
		button_print_in_screen.bind("<ButtonRelease>",fun_sprint_in_screen)
		button_print_in_screen.pack(side="left")
		Label(self.add_row(MPE,bx=150),text="="*500).place(x=0,y=0,anchor="nw") #end
	def GS_IO(self):
		#Loader
		IO=self.add_menu("Database Storage",submenu=1)
		self.pars["IO"]={}
		self.add_row(IO) #skip row (empty row)
		self.add_title(IO,"After-Analysis Data Storage")
		Label(self.add_row(IO,bx=150),text="="*500).place(x=0,y=0,anchor="nw") #begin
		box=self.add_row(IO,bx=150)
		Label(box,text="Engine",width=20).pack(side="left")
		Label(box,text="|").pack(side="left")
		IO_ENG=ttk.Combobox(box,width=18,height=5,values=list(self.pageargs["Infobj"].Config["IO"].keys()))
		if self.pageargs["Infobj"].Default_IO in self.pageargs["Infobj"].Config["IO"]:
			IO_ENG.set(self.pageargs["Infobj"].Default_IO)
		else:
			IO_ENG.set(list(self.pageargs["Infobj"].Config["IO"].keys())[0])
		IO_ENG.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_Loader_ENG(event):
			self.pars["IO"]["Engine"]=IO_ENG.get()
			self.tkobj.io_recv("Updated IO-Engine to",self.pars["IO"]["Engine"])
		button_IO_ENG=Button(box,text="Update",width=5)
		button_IO_ENG.bind("<ButtonRelease>",fun_Loader_ENG)
		button_IO_ENG.pack(side="left")
		self.pars["IO"]["Engine"]=IO_ENG.get()
		Label(self.add_row(IO,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(IO,bx=150)
		Label(box,text="Project name",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		filename=Entry(box,width=15)
		filename.pack(side="left")
		def fun_lfd(event):
			folder_path=filedialog.asksaveasfilename()
			if folder_path!="":
				filename.delete(0,"end")
				filename.insert(0,folder_path)
		button=Button(box,text="Select",width=5)
		button.bind("<ButtonRelease>",fun_lfd)
		button.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_filename(event):
			if filename.get()!="":
				self.pars["IO"]["filename"]=filename.get()
				self.tkobj.io_recv("Updated IO-filename to",self.pars["IO"]["filename"])
			else:
				if "filename" in self.pars["IO"].keys():
					del(self.pars["IO"])
		button_filename=Button(box,text="Update",width=5)
		button_filename.bind("<ButtonRelease>",fun_filename)
		button_filename.pack(side="left")
		if filename.get()!="":
			self.pars["IO"]["filename"]=filename.get()
		Label(self.add_row(IO,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(IO,bx=150)
		Label(box,text="Drop Buffer",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		DropBuffer=ttk.Combobox(box,width=18,height=2,values=[True,False])
		DropBuffer.set(str(True))
		DropBuffer.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_DropBuffer(event): 
			self.pars["IO"]["DropBuffer"]=True if DropBuffer.get()=="True" else False
			self.tkobj.io_recv("Updated IO-DropBuffer to",self.pars["IO"]["DropBuffer"])
		self.pars["IO"]["DropBuffer"]=True if DropBuffer.get()=="True" else False
		button_DropBuffer=Button(box,text="Update",width=5)
		button_DropBuffer.bind("<ButtonRelease>",fun_DropBuffer)
		button_DropBuffer.pack(side="left")
		Label(self.add_row(IO,bx=150),text="="*500).place(x=0,y=0,anchor="nw") #end
	def GS_Loader(self):
		#Loader
		Loader=self.add_menu("Simulation Data Loader",submenu=1)
		self.pars["Loader"]={}
		self.add_row(Loader) #skip row (empty row)
		self.add_title(Loader,"Simulation Data Loader")
		Label(self.add_row(Loader,bx=150),text="="*500).place(x=0,y=0,anchor="nw") #begin
		box=self.add_row(Loader,bx=150)
		Label(box,text="Type",width=20).pack(side="left")
		Label(box,text="|").pack(side="left")
		Loader_ENG=ttk.Combobox(box,width=18,height=5,values=list(self.pageargs["Infobj"].Config["Loader"].keys()))
		Loader_ENG.set(list(self.pageargs["Infobj"].Config["Loader"].keys())[0])
		Loader_ENG.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_Loader_ENG(event):
			self.pars["Loader"]["Engine"]=Loader_ENG.get()
			self.tkobj.io_recv("Updated Loader-Engine to",self.pars["Loader"]["Engine"])
		button_fun_Loader_ENG=Button(box,text="Update",width=5)
		button_fun_Loader_ENG.bind("<ButtonRelease>",fun_Loader_ENG)
		button_fun_Loader_ENG.pack(side="left")
		self.pars["Loader"]["Engine"]=Loader_ENG.get()
		Label(self.add_row(Loader,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(Loader,bx=150)
		Label(box,text="File Dir",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		Loader_FileDir=Entry(box,width=15)
		Loader_FileDir.insert(0,os.getcwd())
		Loader_FileDir.pack(side="left")
		def fun_lfd(event):
			folder_path=filedialog.askdirectory()
			if folder_path!="":
				Loader_FileDir.delete(0,"end")
				Loader_FileDir.insert(0,folder_path)
		button=Button(box,text="Select",width=5)
		button.bind("<ButtonRelease>",fun_lfd)
		button.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_Loader_FileDir(event):
			self.pars["Loader"]["FileDir"]=Loader_FileDir.get()
			self.tkobj.io_recv("Updated Loader-FileDir to",self.pars["Loader"]["FileDir"])
		button_Loader_FileDir=Button(box,text="Update",width=5)
		button_Loader_FileDir.bind("<ButtonRelease>",fun_Loader_FileDir)
		button_Loader_FileDir.pack(side="left")
		self.pars["Loader"]["FileDir"]=Loader_FileDir.get()
		Label(self.add_row(Loader,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(Loader,bx=150)
		Label(box,text="File type",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		Loader_Filetype=Entry(box,width=20)
		Loader_Filetype.insert(0,"dbl")
		Loader_Filetype.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_Loader_Filetype(event):
			self.pars["Loader"]["Filetype"]=Loader_Filetype.get()
			self.tkobj.io_recv("Updated Loader-Filetype to",self.pars["Loader"]["Filetype"])
		button_Loader_Filetype=Button(box,text="Update",width=5)
		button_Loader_Filetype.bind("<ButtonRelease>",fun_Loader_Filetype)
		button_Loader_Filetype.pack(side="left")
		self.pars["Loader"]["FileType"]=Loader_Filetype.get()
		Label(self.add_row(Loader,bx=150),text="-"*500).place(x=0,y=0,anchor="nw") #next
		box=self.add_row(Loader,bx=150)
		Label(box,text="Interval",width=20).pack(side="left")
		Label(box,text="| ").pack(side="left")
		Loader_InterVal=Entry(box,width=20)
		Loader_InterVal.insert(0,1)
		Loader_InterVal.pack(side="left")
		Label(box,text="| ").pack(side="left")
		def fun_Loader_InterVal(event):
			self.pars["Loader"]["Interval"]=int(Loader_InterVal.get())
			self.tkobj.io_recv("Updated Loader-Interval to",self.pars["Loader"]["Interval"])
		button_Loader_InterVal=Button(box,text="Update",width=5)
		button_Loader_InterVal.bind("<ButtonRelease>",fun_Loader_InterVal)
		button_Loader_InterVal.pack(side="left")
		self.pars["Loader"]["Interval"]=int(Loader_InterVal.get())
		Label(self.add_row(Loader,bx=150),text="="*500).place(x=0,y=0,anchor="nw") #end
	def A_Parameters(self):
		#Parameters
		Parameters=self.add_menu("Parameters",submenu=1)
		self.pars["Update"]={}
		self.add_row(Parameters) #skip row (empty row)
		self.add_title(Parameters,"Parameters")
		Label(self.add_row(Parameters,bx=190),text="="*500).place(x=0,y=0,anchor="nw") #begin
		box=self.add_row(Parameters,bx=190)
		Label(box,text="Parameter Name",width=20,fg="green").pack(side="left")
		Label(box,text="| ").pack(side="left")
		Label(box,text="Parameter Value",width=20,fg="green").pack(side="left")
		Label(box,text="| ").pack(side="left")
		Label(box,text="Opera",width=5,fg="green").pack(side="left")
		keys=set(self.pageargs["Infobj"].testdb.data[0].quantities.keys())-set(("vx1","vx2","vx3","SimTime","geometry","rho","prs","Bx1","Bx2","Bx3","output","logfile"))
		def fun(box,entry1,entry2,button,value=None):
			def fun_del(event):
				try:
					del(self.pars["Update"][entry1.get()])
					self.tkobj.io_recv("Delete parameter",entry1.get())
				except:
					self.tkobj.io_recv("Warning: parameter",entry1.get(),"dosen't exist!!!",color="blue")
				entry1.config(state="normal")
				entry2.config(state="normal")
				button.config(text="Save")
				button.bind("<ButtonRelease>",fun_cre)
			def fun_cre(event):
				sttr=str_clean(entry1.get())
				entry1.delete(0,"end")
				entry1.insert(0,sttr)
				sttr=str_clean(entry2.get())
				entry2.delete(0,"end")
				entry2.insert(0,sttr)
				if entry1.get()!="":
					try:
						self.pars["Update"][entry1.get()]=float(entry2.get())
					except:
						self.pars["Update"][entry1.get()]=entry2.get()
					self.tkobj.io_recv("Add parameter",entry1.get(),"to",self.pars["Update"][entry1.get()])
					entry1.config(state="readonly")
					entry2.config(state="readonly")
					button.config(text="Modify")
					button.bind("<ButtonRelease>",fun_del)
			if entry1.get()!="":# normal
				if value!=None:
					entry2.insert(0,str(value))
				else:
					try:
						entry2.insert(0,str(self.pageargs["Infobj"].testdb.data[0].quantities[entry1.get()]))
					except Exception as err:
						print(err)
				try:
					self.pars["Update"][entry1.get()]=float(entry2.get())
				except:
					self.pars["Update"][entry1.get()]=entry2.get()
				entry1.config(state="readonly")
				entry2.config(state="readonly")
				button.config(text="Modify")
				button.bind("<ButtonRelease>",fun_del)
			else:
				button.config(text="Save")
				button.bind("<ButtonRelease>",fun_cre)
		
		for i in range(self.parmax):
			Label(self.add_row(Parameters,bx=190),text="-"*500).place(x=0,y=0,anchor="nw") #next
			box=self.add_row(Parameters,bx=190)
			entry1=Entry(box,width=20)
			if i < len(keys):
				entry1.insert(0,list(keys)[i])
			entry1.pack(side="left")
			Label(box,text="| ").pack(side="left")
			entry2=Entry(box,width=20)
			entry2.pack(side="left")
			Label(box,text="| ").pack(side="left")
			button=Button(box,width=5)
			button.pack(side="left")
			fun(box,entry1,entry2,button)
		Label(self.add_row(Parameters,bx=190),text="="*500).place(x=0,y=0,anchor="nw") #end
	def A_Quantities(self):
		Quantities=self.add_menu("Quantities",submenu=1)
		self.add_row(Quantities) #skip row (empty row)
		self.add_title(Quantities,"Quantities")
		Label(self.add_row(Quantities,bx=20),text="="*500).place(x=0,y=0,anchor="nw") #begin
		box=self.add_row(Quantities,bx=20)
		Label(box,text="Approach",width=20,fg="green").pack(side="left")
		Label(box,text="| ").pack(side="left")
		Label(box,text="Target Quantity",width=20,fg="green").pack(side="left")
		Label(box,text="| ").pack(side="left")
		Label(box,text="Saved Result",width=20,fg="green").pack(side="left")
		Label(box,text="| ").pack(side="left")
		Label(box,text="Arguments",width=20,fg="green").pack(side="left")
		Label(box,text="| ").pack(side="left")
		Label(box,text="Opera",width=5,fg="green").pack(side="left")
		operationlist=list(self.pageargs["Infobj"].Config["AnalysisTool"].keys())+["Gradient","Divergence","Harmonic","Mean","Radial"]
		def get_result(qtname,qto):
			result=""
			try:
				result=self.pageargs["Infobj"].Config["AnalysisTool"][qto].result(qtname,qto)
				if len(result)==1:
					result=result[0]
				else:
					result=str(result)
			except:
				if qto in ("Gradient","Divergence"):
					result=self.pageargs["Infobj"].Config["AnalysisLib"]["Differential"].result(qtname,qto)
					if len(result)==1:
						result=result[0]
					else:
						result=str(result)
				elif qto=="Harmonic":
					result=self.pageargs["Infobj"].Config["AnalysisLib"]["Harmonic_src"].result(qtname,qto)
					if len(result)==1:
						result=result[0]
					else:
						result=str(result)
				elif qto=="Mean":
					result=self.pageargs["Infobj"].Config["AnalysisLib"]["Mean_src"].result(qtname,qto)
					if len(result)==1:
						result=result[0]
					else:
						result=str(result)
				elif qto=="Radial":
					result=self.pageargs["Infobj"].Config["AnalysisLib"]["Radial_src"].result(qtname,qto)
					if len(result)==1:
						result=result[0]
					else:
						result=str(result)
			return strc_vva(result)
		def strc_vva(result):
			return result.replace("(","").replace(")","").replace("[","").replace("]","").replace("\'","").replace("\"","")
		def fun(box,cmbox1,entry2,entry3,entry4,button5,button6):
			def fun_set(event):
				entry3.delete(0,"end")
				ss=""
				#for i in [get_result(i,cmbox1.get()) for i in entry2.get().split(",")]:
				#	ss+=i.replace(" ","")+","
				#if ss!="":
				#	ss=ss[:-1]
				ss=get_result(entry2.get().replace(" ",""),cmbox1.get()).replace(" ","")
				entry3.insert(0,ss)
			def fun_cre(event):
				sttr=str_clean(entry2.get())
				entry2.delete(0,"end")
				entry2.insert(0,sttr)
				sttr=str_clean(entry4.get())
				entry4.delete(0,"end")
				entry4.insert(0,sttr)
				sttr=str_clean(entry3.get())
				entry3.delete(0,"end")
				entry3.insert(0,sttr)
				sttr=str_clean(entry4.get())
				entry4.delete(0,"end")
				entry4.insert(0,sttr)
				if cmbox1.get()!="":
					self.pars[cmbox1.get()]={}
					if strc_vva(entry2.get())!="":
						self.pars[cmbox1.get()]["quantity_name"]=strc_vva(entry2.get())
					if strc_vva(entry3.get())!="":
						self.pars[cmbox1.get()]["result"]=strc_vva(entry3.get())
					for i in entry4.get().split("@"):
						if i!="" and len(i.split("="))==2:
							qu,qv=i.split("=")
							self.pars[cmbox1.get()][qu]=qv.replace(" ","")
					self.tkobj.io_recv("Add approach",cmbox1.get(),"@",self.pars[cmbox1.get()])
					cmbox1.config(state="disable")
					entry2.config(state="readonly")
					entry3.config(state="readonly")
					entry4.config(state="readonly")
					button6.config(text="Modify")
					button6.bind("<ButtonRelease>",fun_del)
			def fun_del(event):
				try:
					del(self.pars[cmbox1.get()])
					self.tkobj.io_recv("Delete approach",cmbox1.get())
				except:
					self.tkobj.io_recv("Warning: approach",cmbox1.get(),"dosen't exist!!!",color="blue")
				cmbox1.config(state="normal")
				entry2.config(state="normal")
				entry3.config(state="normal")
				entry4.config(state="normal")
				button6.config(text="Save")
				button6.bind("<ButtonRelease>",fun_cre)
			button5.bind("<ButtonRelease>",fun_set)
			button6.bind("<ButtonRelease>",fun_cre)
		for i in range(self.parmax):
			Label(self.add_row(Quantities,bx=20),text="-"*500).place(x=0,y=0,anchor="nw") #next
			box=self.add_row(Quantities,bx=20)
			cmbox1=ttk.Combobox(box,width=18,height=len(operationlist) if len(operationlist)<=10 else 10,values=operationlist)
			cmbox1.pack(side="left")
			Label(box,text="| ").pack(side="left")
			entry2=Entry(box,width=20)
			entry2.pack(side="left")
			Label(box,text="| ").pack(side="left")
			entry3=Entry(box,width=14)
			entry3.pack(side="left")
			button5=Button(box,text="Get",width=5)
			button5.pack(side="left")
			Label(box,text="| ").pack(side="left")
			entry4=Entry(box,width=20)
			entry4.pack(side="left")
			Label(box,text="| ").pack(side="left")
			button6=Button(box,text="Save",width=5)
			button6.pack(side="left")
			fun(box,cmbox1,entry2,entry3,entry4,button5,button6)
		Label(self.add_row(Quantities,bx=20),text="="*500).place(x=0,y=0,anchor="nw") #end
	def save(self):
		save=self.add_menu("Save Configuration")
		self.add_row(save) #skip row (empty row)
		self.add_title(save,"Save Configuration")
		box=self.add_row(save,bx=300)
		def Test(event):
			self.tkobj.io_recv("Get Parameters:")
			for i in self.pars.keys():
				self.tkobj.io_recv(i,":",self.pars[i])
			self.tkobj.io_recv(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			self.pageargs["Infobj"].testdb.update(**self.pars["Update"])
			self.tkobj.io_recv("Ready for testing..............")
			ext_tools=("Harmonic","Mean","Gradient","Divergence","Radial")
			for i in self.pars.keys():
				if i in self.pageargs["Infobj"].Config["AnalysisTool"].keys():
					anaf=self.pageargs["Infobj"].Config["AnalysisTool"][i].get
				elif i in ext_tools:
					anaf=self.pageargs["Infobj"].Config["AnalysisLib"][i]
				else:
					continue
				pas=copy.deepcopy(self.pars[i])
				pas.update({"info":i})
				for j in pas.keys():
					pas[j]=retype_string(pas[j])

				#if "quantity_name" in pas.keys():
				#	pas["quantity_name"]=pas["quantity_name"].split(",")
				self.pageargs["Infobj"].testdb.analysis_data(anaf,**pas)
				if "result" in pas.keys():
					self.pageargs["Infobj"].testdb.check_quantities(pas["result"])
			self.tkobj.io_recv("Operation completed",color="green")
		button=Button(box,text="Test",width=5)
		button.pack(side="left")
		button.bind("<ButtonRelease>",Test)
		def Save(event):
			file=filedialog.asksaveasfile()
			if file!=None:
				self.tkobj.io_recv("Collecting Parameters:")
				for i in self.pars.keys():
					self.tkobj.io_recv(i,":",self.pars[i])
				self.tkobj.io_recv("Saving to file @",file)
				strc="#ShockFinder Analyzing Configurations\n"
				try:
					strc+="#Built at "+Time.now()+"\n"
				except:
					pass
				strc+="#Wacmk.cn/com\n"
				strc+="#https://www.github.com/wacmkxiaoyi/shockfinder\n\n"
				#write MPE
				for key in ("MultiprocessEngine","IO","Loader","Update"):#maintain sequency
					if key in self.pars.keys():
						strc+=key
						dd=True
						for i in self.pars[key].keys():
							strc+="@"+i+"="+str(self.pars[key][i])
							dd=False
						if dd:
							strc+="@"
						strc+="\n"
				for i in self.pars.keys():
					if i not in ("MultiprocessEngine","IO","Loader","Update"):
						strc+=i
						dd=True
						for j in self.pars[i].keys():
							strc+="@"+j+"="+str(self.pars[i][j])
							dd=False
						if dd:
							strc+="@"
						strc+="\n"
				file.writelines(strc)
				file.close()
				self.tkobj.io_recv("Operation completed",color="green")
		Label(box,width=10).pack(side="left")
		button=Button(box,text="Save",width=5)
		button.pack(side="left")
		button.bind("<ButtonRelease>",Save)
	def initial(self):
		self.save()
		self.set_image(self.img["logo"])
		self.add_useless_menu("Global Settings↓")
		self.GS_MPE()
		self.GS_IO()
		self.GS_Loader()
		self.add_useless_menu("Analysis↓")
		self.A_Parameters()
		self.A_Quantities()


#File type: <Function>
#By Junxiang H., 2023/07/03
#wacmk.com/cn Tech. Supp.
try:
	import ShockFinder.Addon.Painter.Basic as Basic
except:
	import Basic
import pandas as pd,numpy as np
def CreateLine(**args):
	x=Basic.get_par(args,"x")
	y=Basic.get_par(args,"y")
	z=Basic.get_par(args,"z")
	if type(x) == type(None) or type(y) == type(None):
		return None
	label=Basic.get_par(args,"label","")
	color=Basic.get_par(args,"color")
	linestyle=Basic.get_par(args,"linestyle","-")
	if type(z)!=type(None):
		line={
			"x":x,
			"y":y,
			"z":z}
	else:
		line={
			"x":x,
			"y":y}
	lineinfo={
			"label":None if label==None else Basic.CharUndecode(label),
			"color":Basic.set_None(color),
			"linestyle":linestyle
		}
	for i in args.keys():
		if i not in lineinfo.keys() and i not in line.keys():
			lineinfo[i]=args[i]
	return (line,lineinfo)

def FormatLine(line,lineinfo,lineid):
	lineframe=pd.DataFrame(line,dtype=float)
	lineinfoframe=pd.DataFrame(lineinfo,index=[lineid])
	return (lineframe,lineinfoframe)

def DecodeLine(lineframe,lineinfoframe):
	data=dict(zip(["x","y","z"],[np.array(list(lineframe.to_dict()[i].values())) for i in lineframe.keys()])) 
	data.update(lineinfoframe.to_dict())
	return CreateLine(**data)

def info():
	print("Module:",__file__)


if __name__=="__main__":
	x=np.arange(0,100,0.1)
	line0,lineinfo0=CreateLine(x=x,y=x**2)
	line0frame,line0infoframe=FormatLine(line0,lineinfo0,0)
	#print(line0frame,"\n",line0infoframe)
	print(DecodeLine(line0frame,line0infoframe.loc[0]))


#File type: extension <Module> set
#By Junxiang H., 2023/07/1
#wacmk.com/cn Tech. Supp.

#This script updates automaticly! Do not Modify!
#Update time:2023-10-24 06:22:13

GUI={}
try:
	try:
		import ShockFinder.Addon.GUI.XUI as XUI
	except Exception as err:
		print("XUIin ShockFinder dir importing error, reimporting in current dir")
		import XUI
	GUI["XUI"]=XUI
except Exception as err:
	print("Module: XUI import failure:",err)

if __name__=="__main__":
	print("Testing Model:",__file__)
	print("GUI:")
	for i in GUI.keys():
		print(i,":",GUI[i])
		print("\t",GUI[i].show)


try:
	import ShockFinder.Addon.Painter.Basic as Basic
except:
	import Basic
import pandas as pd,numpy as np,copy

def CreateScatter(**args):
	x=Basic.get_par(args,"x")
	y=Basic.get_par(args,"y")
	z=Basic.get_par(args,"z")
	v=Basic.get_par(args,"v")
	if type(x) == type(None) or type(y) == type(None) or type(v) == type(None):
		return None
	scatter={
		"x":x,
		"y":y,
		"z":v,
		"v":z
	}
	label=Basic.get_par(args,"label","")
	if type(z)!=type(None):
		Scatter={
			"x":x,
			"y":y,
			"v":v,
			"z":z}
	else:
		Scatter={
			"x":x,
			"y":y,
			"v":v}
	Scatterinfo={
			#"label":Basic.CharUndecode(label)
		}
	for i in args.keys():
		if i not in Scatterinfo.keys() and i not in Scatter.keys():
			Scatterinfo[i]=args[i]
	return (Scatter,Scatterinfo)

def info():
	print("Module:",__file__)


#Junxiang H. 2023.07.09
import os
from XenonUI.XUIlib.imgtool import add_image
from XenonUI.XUIlib.page import *
from ShockFinder.Config import ShockFinderVersion
from ShockFinder.Addon.GUI.XUIlib.image.config import Image_SF7
from tkinter import *
class page(page):
	img={
		"main_logo":Image_SF7,
		"logo":Image_H
	}
	def page_default(self):	
		self.default_page_smbindex=self.add_menu("ShockFinder "+str(ShockFinderVersion))
		self.default_page_roindex=self.add_item(self.default_page_smbindex,self.ro_height)
		add_image(self.tkobj.ro_item[self.fmbindex][self.default_page_smbindex][self.default_page_roindex][1],self.img["main_logo"],width=self.ro_width,height=self.ro_height)
	def page_Document(self):
		intro=self.add_menu("Readme")
		self.add_item(intro,50)
		self.add_title(intro,"Document of ShockFinder please see:",fg="red")
		self.add_title(intro,"https://www.github/wacmkxiaoyi/shockfinder",fontsize=10)
	def page_author_information(self):
		aui=self.add_menu("Author information")
		self.add_item(aui,50)
		self.add_title(aui,"Physics Algorithm",fontsize=12)
		self.add_title(aui,"C. B. Singh & Junxiang H.",fg="black",height=100)
		self.add_title(aui,"GUI Frame Design/IO Engine/DBMS",fontsize=12)
		self.add_title(aui,"Junxiang H. (XUIv1.1)",fg="black",height=100)
		self.add_title(aui,"Multi-processes Algorithm/Engine Design",fontsize=12)
		self.add_title(aui,"Junxiang H. & Weihui L. (XMEv3.0)",fg="black")
	def initial(self):
		self.set_image(self.img["logo"])
		self.page_default()
		self.page_author_information()
		self.page_Document()
	def show(self):
		self.tkobj.open_smb(None,self.fmbindex)
		self.tkobj.open_ro(None,self.fmbindex,self.default_page_smbindex)